"""
SRA-MACIE-07: All active member accounts have relationship with delegated admin account enabled.
"""
from typing import List, Dict, Any, Set
from sraverify.services.macie.base import MacieCheck
from sraverify.core.logging import logger


class SRA_MACIE_07(MacieCheck):
    """Check if all active member accounts have relationship with delegated admin account enabled."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-MACIE-07"
        self.check_name = "All active member accounts have relationship with delegated admin account enabled"
        self.description = (
            "This check verifies whether all active members accounts of the AWS Organization have Macie member relationship "
            "enabled with Macie delegated admin account. Amazon Macie is a data security service that discovers sensitive data "
            "by using machine learning and pattern matching, provides visibility into data security risks, and enables automated "
            "protection against those risks."
        )
        self.severity = "HIGH"
        self.account_type = "audit"
        self.check_logic = "Check runs organizations list-accounts AND macie2 list-members. Check PASS if macie2 list-members includes all members of the AWS organization minus the audit account."
        self.resource_type = "AWS::Macie::Session"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check if audit accounts are provided
        audit_accounts = []
        if hasattr(self, '_audit_accounts') and self._audit_accounts:
            audit_accounts = self._audit_accounts
        
        for region in self.regions:
            # Get organization members using the base class method with caching
            org_members = self.get_organization_members(region)
            
            # Check if the API call was successful
            if not org_members:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"organization/{self.account_id}",
                        checked_value="All active member accounts have Macie relationship enabled",
                        actual_value="Failed to retrieve AWS Organization members",
                        remediation="Ensure you have the necessary permissions to call the Organizations ListAccounts API"
                    )
                )
                continue
            
            # Get Macie members using the base class method with caching
            macie_members = self.get_macie_members(region)
            
            # Check if the API call was successful
            if macie_members is None:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}",
                        checked_value="All active member accounts have Macie relationship enabled",
                        actual_value="Failed to retrieve Macie members",
                        remediation="Ensure you have the necessary permissions to call the Macie ListMembers API"
                    )
                )
                continue
            
            # Filter active organization members
            active_org_members = [
                member for member in org_members 
                if member.get('Status') == 'ACTIVE'
            ]
            
            # Create sets of account IDs for comparison
            active_org_account_ids = {member.get('Id') for member in active_org_members}
            macie_member_account_ids = {member.get('accountId') for member in macie_members}
            
            # Remove the current account (delegated admin) from the set of accounts to check
            if self.account_id in active_org_account_ids:
                active_org_account_ids.remove(self.account_id)
            
            # Remove audit accounts from the set of accounts to check if provided
            for audit_account in audit_accounts:
                if audit_account in active_org_account_ids:
                    active_org_account_ids.remove(audit_account)
            
            # Find accounts that are not Macie members
            missing_accounts = active_org_account_ids - macie_member_account_ids
            
            # Find accounts that are Macie members but not enabled
            not_enabled_accounts = []
            for member in macie_members:
                if member.get('accountId') in active_org_account_ids and member.get('relationshipStatus') != 'Enabled':
                    not_enabled_accounts.append(member.get('accountId'))
            
            if not missing_accounts and not not_enabled_accounts:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="All active member accounts have Macie relationship enabled",
                        actual_value=f"All {len(active_org_account_ids)} active member accounts have Macie relationship enabled in region {region}",
                        remediation="No remediation needed"
                    )
                )
            else:
                missing_accounts_str = ", ".join(missing_accounts) if missing_accounts else "None"
                not_enabled_accounts_str = ", ".join(not_enabled_accounts) if not_enabled_accounts else "None"
                
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="All active member accounts have Macie relationship enabled",
                        actual_value=(
                            f"Not all active member accounts have Macie relationship enabled in region {region}. "
                            f"Missing accounts: {missing_accounts_str}. "
                            f"Accounts with relationship not enabled: {not_enabled_accounts_str}."
                        ),
                        remediation=(
                            f"Enable Macie for all member accounts in region {region} using the AWS CLI command: "
                            f"aws macie2 create-member --account account_details --region {region}"
                        )
                    )
                )
        
        return findings



"""
SRA-INSPECTOR-06: Inspector Delegated Admin Account is the Audit Account.
"""
from typing import List, Dict, Any
from sraverify.services.inspector.base import InspectorCheck
from sraverify.core.logging import logger


class SRA_INSPECTOR_06(InspectorCheck):
    """Check if Inspector delegated admin account is the audit account."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-INSPECTOR-06"
        self.check_name = "Inspector delegated admin account is the audit account"
        self.account_type = "management"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether Inspector delegated admin account is the audit account of your AWS organization. "
            "Audit account is dedicated to operating security services, monitoring AWS accounts, and automating security "
            "alerting and response. Inspector provides vulnerability management service."
        )
        self.check_logic = (
            "Check runs inspector2 get-delegated-admin-account. PASS if delegated admin is the Audit account "
            "specified by flag --audit-account"
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        
        # Check each region separately
        for region in self.regions:
            # Get delegated admin account for this region
            delegated_admin_response = self.get_delegated_admin(region)
            delegated_admin = delegated_admin_response.get('delegatedAdmin', {})
            delegated_admin_id = delegated_admin.get('accountId')
            
            # If no delegated admin is configured, report a failure
            if not delegated_admin_id:
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is the audit account",
                        actual_value="No delegated admin account is configured",
                        remediation=(
                            "Configure a delegated admin account for Inspector using the AWS Console or CLI command: "
                            f"aws organizations register-delegated-administrator --account-id <AUDIT_ACCOUNT_ID> "
                            f"--service-principal inspector2.amazonaws.com --region {region}"
                        )
                    )
                )
                continue
            
            # Check if audit_accounts is provided via _audit_accounts (new attribute name)
            audit_accounts = []
            if hasattr(self, '_audit_accounts') and self._audit_accounts:
                audit_accounts = self._audit_accounts
            # For backward compatibility, also check the old attribute name
            elif hasattr(self, 'audit_accounts') and self.audit_accounts:
                audit_accounts = self.audit_accounts
                
            if not audit_accounts:
                self.findings.append(
                    self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is the audit account",
                        actual_value=f"Delegated admin account is {delegated_admin_id}, but no audit account was specified for comparison",
                        remediation="Run the check with the --audit-account parameter to specify the audit account"
                    )
                )
                continue
            
            # Check if the delegated admin is one of the audit accounts
            if delegated_admin_id in audit_accounts:
                self.findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is the audit account",
                        actual_value=f"Inspector delegated administrator (Account: {delegated_admin_id}) "
                                   f"matches one of the specified Audit accounts {', '.join(audit_accounts)}",
                        remediation="No remediation needed"
                    )
                )
            else:
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is the audit account",
                        actual_value=f"Inspector delegated administrator (Account: {delegated_admin_id}) "
                                   f"does not match any of the specified Audit accounts ({', '.join(audit_accounts)})",
                        remediation=(
                            "Update the delegated admin account to be the audit account using the AWS Console or CLI commands: "
                            f"1. aws organizations deregister-delegated-administrator --account-id {delegated_admin_id} "
                            f"--service-principal inspector2.amazonaws.com --region {region}\n"
                            f"2. aws organizations register-delegated-administrator --account-id {audit_accounts[0]} "
                            f"--service-principal inspector2.amazonaws.com --region {region}"
                        )
                    )
                )
        
        return self.findings



"""
Cloudtrail security checks.
"""
from sraverify.services.cloudtrail.checks.sra_cloudtrail_01 import SRA_CLOUDTRAIL_01
from sraverify.services.cloudtrail.checks.sra_cloudtrail_02 import SRA_CLOUDTRAIL_02
from sraverify.services.cloudtrail.checks.sra_cloudtrail_03 import SRA_CLOUDTRAIL_03
from sraverify.services.cloudtrail.checks.sra_cloudtrail_04 import SRA_CLOUDTRAIL_04
from sraverify.services.cloudtrail.checks.sra_cloudtrail_05 import SRA_CLOUDTRAIL_05
from sraverify.services.cloudtrail.checks.sra_cloudtrail_06 import SRA_CLOUDTRAIL_06
from sraverify.services.cloudtrail.checks.sra_cloudtrail_07 import SRA_CLOUDTRAIL_07
from sraverify.services.cloudtrail.checks.sra_cloudtrail_08 import SRA_CLOUDTRAIL_08
from sraverify.services.cloudtrail.checks.sra_cloudtrail_09 import SRA_CLOUDTRAIL_09
from sraverify.services.cloudtrail.checks.sra_cloudtrail_10 import SRA_CLOUDTRAIL_10
from sraverify.services.cloudtrail.checks.sra_cloudtrail_11 import SRA_CLOUDTRAIL_11
from sraverify.services.cloudtrail.checks.sra_cloudtrail_12 import SRA_CLOUDTRAIL_12
from sraverify.services.cloudtrail.checks.sra_cloudtrail_13 import SRA_CLOUDTRAIL_13

# Register checks
CHECKS = {
    "SRA-CLOUDTRAIL-01": SRA_CLOUDTRAIL_01,
    "SRA-CLOUDTRAIL-02": SRA_CLOUDTRAIL_02,
    "SRA-CLOUDTRAIL-03": SRA_CLOUDTRAIL_03,
    "SRA-CLOUDTRAIL-04": SRA_CLOUDTRAIL_04,
    "SRA-CLOUDTRAIL-05": SRA_CLOUDTRAIL_05,
    "SRA-CLOUDTRAIL-06": SRA_CLOUDTRAIL_06,
    "SRA-CLOUDTRAIL-07": SRA_CLOUDTRAIL_07,
    "SRA-CLOUDTRAIL-08": SRA_CLOUDTRAIL_08,
    "SRA-CLOUDTRAIL-09": SRA_CLOUDTRAIL_09,
    "SRA-CLOUDTRAIL-10": SRA_CLOUDTRAIL_10,
    "SRA-CLOUDTRAIL-11": SRA_CLOUDTRAIL_11,
    "SRA-CLOUDTRAIL-12": SRA_CLOUDTRAIL_12,
    "SRA-CLOUDTRAIL-13": SRA_CLOUDTRAIL_13,
}



"""
SRA-CLOUDTRAIL-13: CloudTrail Delegated Administrator is the Audit Account.
"""
from typing import List, Dict, Any
from sraverify.services.cloudtrail.base import CloudTrailCheck
from sraverify.core.logging import logger


class SRA_CLOUDTRAIL_13(CloudTrailCheck):
    """Check if CloudTrail delegated administrator is the Audit account."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CLOUDTRAIL-13"
        self.check_name = "The audit account is the Delegated Administrator set for CloudTrail"
        self.account_type = "management"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether CloudTrail delegated admin account is the audit account of your AWS organization. "
            "Audit account is dedicated to operating security services, monitoring AWS accounts, and "
            "automating security alerting and response. CloudTrail helps monitor API activities across "
            "all your AWS accounts and regions."
        )
        self.check_logic = (
            "Check if the delegated administrator account matches any of the specified Audit account IDs."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Get delegated administrators for CloudTrail
        # This will use the cache if available or make API calls if needed
        delegated_admins = self.get_delegated_administrators()
        
        if not delegated_admins:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=f"organization/{self.account_id}",
                    checked_value="CloudTrail delegated administrator is an Audit account",
                    actual_value="No delegated administrator configured for CloudTrail",
                    remediation=(
                        "Register an Audit account as delegated administrator for CloudTrail using the AWS CLI command: "
                        "aws organizations register-delegated-administrator "
                        "--account-id AUDIT_ACCOUNT_ID --service-principal cloudtrail.amazonaws.com"
                    )
                )
            )
            return findings
        
        # Check if audit_accounts is provided via _audit_accounts attribute
        audit_accounts = []
        if hasattr(self, '_audit_accounts') and self._audit_accounts:
            audit_accounts = self._audit_accounts
        
        if not audit_accounts:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id=f"organization/{self.account_id}",
                    checked_value="CloudTrail delegated administrator is an Audit account",
                    actual_value="Audit Account ID not provided",
                    remediation="Provide the Audit account IDs using --audit-account flag"
                )
            )
            return findings
        
        # Check if any of the delegated administrators is an Audit account
        for admin in delegated_admins:
            admin_id = admin.get('Id', 'Unknown')
            admin_name = admin.get('Name', 'Unknown')
            
            # Create a resource ID that includes the delegated admin and audit account info
            resource_id = f"cloudtrail arn has delegated administrator set to {admin_id}, audit account is {audit_accounts[0]}, {admin_id in audit_accounts}"
            
            if admin_id in audit_accounts:
                # This delegated admin is in the audit accounts list
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region="global",
                        resource_id=resource_id,
                        checked_value=f"CloudTrail delegated administrator is an Audit account ({', '.join(audit_accounts)})",
                        actual_value=f"CloudTrail delegated administrator {admin_id} ({admin_name}) is an Audit account",
                        remediation="No remediation needed"
                    )
                )
            else:
                # This delegated admin is not in the audit accounts list
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region="global",
                        resource_id=resource_id,
                        checked_value=f"CloudTrail delegated administrator is an Audit account ({', '.join(audit_accounts)})",
                        actual_value=(
                            f"CloudTrail delegated administrator {admin_id} ({admin_name}) "
                            f"is not in the specified Audit accounts ({', '.join(audit_accounts)})"
                        ),
                        remediation=(
                            "Deregister the current delegated administrator and register an Audit account "
                            "as delegated administrator for CloudTrail using the AWS CLI commands: "
                            f"aws organizations deregister-delegated-administrator "
                            f"--account-id {admin_id} --service-principal cloudtrail.amazonaws.com && "
                            "aws organizations register-delegated-administrator "
                            f"--account-id {audit_accounts[0]} --service-principal cloudtrail.amazonaws.com"
                        )
                    )
                )
        
        return findings



"""
Check if Shield Advanced is enabled.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_01(ShieldCheck):
    """Check if Shield Advanced is enabled."""

    def __init__(self):
        """Initialize Shield Advanced enabled check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-01"
        self.check_name = "Shield Advanced is enabled"
        self.description = ("This check verifies that AWS Shield Advanced is enabled. "
                           "Shield Advanced provides enhanced DDoS protection for your AWS resources "
                           "and includes 24/7 access to the AWS DDoS Response Team (DRT).")
        self.severity = "HIGH"
        self.check_logic = "Get Shield subscription state. Check fails if subscription is not active."

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        status = self.get_subscription_status(region)

        if "Error" in status:
            error_code = status["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield Advanced not subscribed",
                    remediation="Enable Shield Advanced subscription in the AWS Shield console"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=status["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif status.get("SubscriptionState") == "ACTIVE":
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id="shield:subscription",
                actual_value="Shield Advanced is active",
                remediation=""
            ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value=f"Subscription state: {status.get('SubscriptionState', 'Unknown')}",
                remediation="Enable Shield Advanced subscription in the AWS Shield console"
            ))

        return self.findings



from sraverify.services.firewallmanager.checks.sra_firewallmanager_01 import SRA_FIREWALLMANAGER_01
from sraverify.services.firewallmanager.checks.sra_firewallmanager_02 import SRA_FIREWALLMANAGER_02
from sraverify.services.firewallmanager.checks.sra_firewallmanager_03 import SRA_FIREWALLMANAGER_03
from sraverify.services.firewallmanager.checks.sra_firewallmanager_04 import SRA_FIREWALLMANAGER_04
from sraverify.services.firewallmanager.checks.sra_firewallmanager_05 import SRA_FIREWALLMANAGER_05
from sraverify.services.firewallmanager.checks.sra_firewallmanager_06 import SRA_FIREWALLMANAGER_06
from sraverify.services.firewallmanager.checks.sra_firewallmanager_07 import SRA_FIREWALLMANAGER_07
from sraverify.services.firewallmanager.checks.sra_firewallmanager_08 import SRA_FIREWALLMANAGER_08
from sraverify.services.firewallmanager.checks.sra_firewallmanager_09 import SRA_FIREWALLMANAGER_09
from sraverify.services.firewallmanager.checks.sra_firewallmanager_10 import SRA_FIREWALLMANAGER_10

CHECKS = {
    "SRA-FIREWALLMANAGER-01": SRA_FIREWALLMANAGER_01,
    "SRA-FIREWALLMANAGER-02": SRA_FIREWALLMANAGER_02,
    "SRA-FIREWALLMANAGER-03": SRA_FIREWALLMANAGER_03,
    "SRA-FIREWALLMANAGER-04": SRA_FIREWALLMANAGER_04,
    "SRA-FIREWALLMANAGER-05": SRA_FIREWALLMANAGER_05,
    "SRA-FIREWALLMANAGER-06": SRA_FIREWALLMANAGER_06,
    "SRA-FIREWALLMANAGER-07": SRA_FIREWALLMANAGER_07,
    "SRA-FIREWALLMANAGER-08": SRA_FIREWALLMANAGER_08,
    "SRA-FIREWALLMANAGER-09": SRA_FIREWALLMANAGER_09,
    "SRA-FIREWALLMANAGER-10": SRA_FIREWALLMANAGER_10,
}



"""SecurityHub service checks."""

from sraverify.services.securityhub.checks.sra_securityhub_01 import SRA_SECURITYHUB_01
from sraverify.services.securityhub.checks.sra_securityhub_02 import SRA_SECURITYHUB_02
from sraverify.services.securityhub.checks.sra_securityhub_03 import SRA_SECURITYHUB_03
from sraverify.services.securityhub.checks.sra_securityhub_04 import SRA_SECURITYHUB_04
from sraverify.services.securityhub.checks.sra_securityhub_05 import SRA_SECURITYHUB_05
from sraverify.services.securityhub.checks.sra_securityhub_06 import SRA_SECURITYHUB_06
from sraverify.services.securityhub.checks.sra_securityhub_07 import SRA_SECURITYHUB_07
from sraverify.services.securityhub.checks.sra_securityhub_08 import SRA_SECURITYHUB_08
from sraverify.services.securityhub.checks.sra_securityhub_09 import SRA_SECURITYHUB_09
from sraverify.services.securityhub.checks.sra_securityhub_10 import SRA_SECURITYHUB_10
from sraverify.services.securityhub.checks.sra_securityhub_11 import SRA_SECURITYHUB_11

CHECKS = {
    "SRA-SECURITYHUB-01": SRA_SECURITYHUB_01,
    "SRA-SECURITYHUB-02": SRA_SECURITYHUB_02,
    "SRA-SECURITYHUB-03": SRA_SECURITYHUB_03,
    "SRA-SECURITYHUB-04": SRA_SECURITYHUB_04,
    "SRA-SECURITYHUB-05": SRA_SECURITYHUB_05,
    "SRA-SECURITYHUB-06": SRA_SECURITYHUB_06,
    "SRA-SECURITYHUB-07": SRA_SECURITYHUB_07,
    "SRA-SECURITYHUB-08": SRA_SECURITYHUB_08,
    "SRA-SECURITYHUB-09": SRA_SECURITYHUB_09,
    "SRA-SECURITYHUB-10": SRA_SECURITYHUB_10,
    "SRA-SECURITYHUB-11": SRA_SECURITYHUB_11,
}



"""
Check if GuardDuty has RDS protection enabled.
"""
from typing import Dict, List, Any
from sraverify.services.guardduty.base import GuardDutyCheck


class SRA_GUARDDUTY_10(GuardDutyCheck):
    """Check if GuardDuty has RDS protection enabled."""

    def __init__(self):
        """Initialize GuardDuty RDS protection check."""
        super().__init__()
        self.check_id = "SRA-GUARDDUTY-10"
        self.check_name = "GuardDuty RDS protection enabled"
        self.description = ("This check verifies that GuardDuty RDS protection is enabled. "
                           "RDS Protection in Amazon GuardDuty analyzes and profiles RDS login activity "
                           "for potential access threats to Amazon Aurora databases and Amazon RDS for PostgreSQL.")
        self.severity = "HIGH"
        self.check_logic = "Get detector details in each Region. Check if RDS protection is enabled in the Features array."
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []        
        # Check all regions
        for region in self.regions:
            detector_id = self.get_detector_id(region)
            
            # Handle regions where we can't access GuardDuty
            if not detector_id:
                findings.append(self.create_finding(
                    status="ERROR", 
                    region=region, 
                    resource_id=f"guardduty:{region}", 
                    actual_value="Unable to access GuardDuty in this region", 
                    remediation="Check permissions or if GuardDuty is supported in this region"
                ))
                continue
                
            # Get detector details
            detector_details = self.get_detector_details(region)
            
            if detector_details:
                # Check if RDS protection is enabled in the Features array
                rds_protection_enabled = False
                features = detector_details.get('Features', [])
                
                for feature in features:
                    if feature.get('Name') == 'RDS_LOGIN_EVENTS' and feature.get('Status') == 'ENABLED':
                        rds_protection_enabled = True
                        break
                
                if rds_protection_enabled:
                    findings.append(self.create_finding(
                        status="PASS", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value="RDS protection is enabled", 
                        remediation=""
                    ))
                else:
                    findings.append(self.create_finding(
                        status="FAIL", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value="RDS protection is not enabled", 
                        remediation=f"Enable RDS protection for GuardDuty in {region} to monitor login activity for potential threats to Aurora and RDS for PostgreSQL databases"
                    ))
            else:
                findings.append(self.create_finding(
                    status="FAIL", 
                    region=region, 
                    resource_id=f"guardduty:{region}:{detector_id}", 
                    actual_value="Unable to retrieve detector details", 
                    remediation="Check GuardDuty permissions and configuration"
                ))
        
        return findings



"""
Audit Manager client for interacting with AWS Audit Manager service.
"""
from typing import Dict, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class AuditManagerClient:
    """Client for interacting with AWS Audit Manager service."""

    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        """
        Initialize Audit Manager client for a specific region.

        Args:
            region: AWS region name
            session: AWS session to use (if None, a new session will be created)
        """
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('auditmanager', region_name=region)

    def get_account_status(self) -> Dict[str, Any]:
        """
        Get the registration status of the account in Audit Manager.

        Returns:
            Dictionary containing status or error information
        """
        try:
            response = self.client.get_account_status()
            return response
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = str(e)
            logger.error(f"Error getting account status in {self.region}: {error_message}")

    def get_organization_admin_account(self) -> Dict[str, Any]:
        """
        Get the delegated administrator account for the organization.

        Returns:
            Dictionary containing admin account info or error information
        """
        try:
            response = self.client.get_organization_admin_account()
            return response
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = str(e)
            
            # Don't log setup required errors as they're handled as FAIL in the check
            if "Please complete AWS Audit Manager setup" not in error_message:
                logger.error(f"Error getting organization admin account in {self.region}: {error_message}")
            
            return {"Error": {"Code": error_code, "Message": error_message}}



"""Check if Security Lake has a delegated administrator."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_14(SecurityLakeCheck):
    """Check if Security Lake has a delegated administrator."""

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.account_type = "management"  # Delegated admin check runs from management account
        self.check_id = "SRA-SECURITYLAKE-14"
        self.check_name = "Security Lake has delegated administrator"
        self.severity = "CRITICAL"
        self.description = (
            "This check verifies whether Security Lake service administration for "
            "the AWS Organization is delegated out from AWS Organization management "
            "account to a member account."
        )
        self.check_logic = (
            "Checks if Security Lake has a delegated administrator configured. "
            "The check passes if at least one delegated administrator is found for the Security Lake service. "
            "The check fails if no delegated administrator is configured."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """Run check."""
        findings = []

        # This is a global check, so we only need to run it once
        # Use the first region just to make the API call
        region = self.regions[0] if self.regions else "us-east-1"
        resource_id = f"arn:aws:organizations::global:delegatedadministrator/securitylake"

        # Get delegated administrators using the base class method
        delegated_admin = self.get_delegated_administrators(region)

        if not delegated_admin:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=resource_id,
                    checked_value="Delegated administrator configured",
                    actual_value="No delegated administrator configured for Security Lake",
                    remediation=(
                        "Configure a delegated administrator for Security Lake. In the AWS Organizations console, "
                        "navigate to Services > Security Lake and delegate administration to a member account. "
                        "Alternatively, use the AWS CLI command: "
                        "aws organizations register-delegated-administrator --service-principal securitylake.amazonaws.com "
                        "--account-id ACCOUNT_ID"
                    )
                )
            )
        else:
            admin_info = delegated_admin[0] if delegated_admin else {}  # Get first admin if exists
            admin_id = admin_info.get('Id', 'Unknown')
            findings.append(
                self.create_finding(
                    status="PASS",
                    region="global",
                    resource_id=resource_id,
                    checked_value="Delegated administrator configured",
                    actual_value=f"Delegated administrator {admin_id} is configured for Security Lake",
                    remediation="No remediation needed"
                )
            )

        return findings



"""
SRA-MACIE-08: Macie AutoEnable configuration is enabled for new member accounts.
"""
from typing import List, Dict, Any
from sraverify.services.macie.base import MacieCheck
from sraverify.core.logging import logger


class SRA_MACIE_08(MacieCheck):
    """Check if Macie AutoEnable configuration is enabled for new member accounts."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-MACIE-08"
        self.check_name = "Macie AutoEnable configuration is enabled for new member accounts"
        self.description = (
            "This check verifies whether auto-enablement configuration for Macie is enabled for member accounts of the AWS Organization. "
            "This ensures that all existing and new member accounts will have Macie monitoring."
        )
        self.severity = "MEDIUM"
        self.account_type = "audit"
        self.check_logic = "Check runs macie2 describe-organization-configuration. PASS if autoenable = True"
        self.resource_type = "AWS::Macie::Session"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        for region in self.regions:
            # Get organization configuration using the base class method with caching
            org_config = self.get_organization_configuration(region)
            
            # Check if the API call was successful
            if not org_config:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="autoEnable: true",
                        actual_value="Failed to retrieve Macie organization configuration",
                        remediation="Ensure Macie is enabled and you have the necessary permissions to call the Macie DescribeOrganizationConfiguration API"
                    )
                )
                continue
            
            # Check if auto-enable is enabled
            auto_enable = org_config.get('autoEnable', False)
            
            if auto_enable:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="autoEnable: true",
                        actual_value=f"Macie AutoEnable configuration is enabled for new member accounts in region {region}",
                        remediation="No remediation needed"
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="autoEnable: true",
                        actual_value=f"Macie AutoEnable configuration is not enabled for new member accounts in region {region}",
                        remediation=(
                            f"Enable Macie AutoEnable configuration for new member accounts in region {region} using the AWS CLI command: "
                            f"aws macie2 update-organization-configuration --auto-enable --region {region}"
                        )
                    )
                )
        
        return findings



"""
SRA-SECURITYHUB-05: Security Hub check.
"""
from typing import List, Dict, Any
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_05(SecurityHubCheck):
    """Check if Security Hub has integrations with findings generating products."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-05"
        self.check_name = "Security Hub integration with findings generating products"
        self.account_type = "audit"  # This check is for the audit account
        self.severity = "MEDIUM"
        self.description = (
            "This check verifies whether Security Hub has expected integration with AWS services "
            "and third party products to ingest security findings."
        )
        self.check_logic = (
            "Check looks in audit account to evaluate see what types of findings are being ingested. "
            "PASS if there are any products listed."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check each region separately
        for region in self.regions:
            # Get enabled products for import in this specific region
            enabled_products = self.get_enabled_products_for_import(region)
            
            resource_id = f"securityhub:integrations/{self.account_id}"
            
            # Check if Security Hub is not enabled (None response)
            if enabled_products is None:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="Security Hub enabled with product integrations",
                        actual_value=f"Security Hub is not enabled in region {region}",
                        remediation=(
                            f"Enable Security Hub in region {region} first. "
                            "Use the AWS CLI command: "
                            f"aws securityhub enable-security-hub --region {region}, "
                            "then configure product integrations."
                        )
                    )
                )
            elif not enabled_products:
                # Security Hub is enabled but no products configured
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="List of products that are enabled in Security Hub",
                        actual_value=f"Security Hub has no enabled security integrations in region {region}",
                        remediation=(
                            "Enable integrations with security findings generating products. In the Security Hub console, "
                            "navigate to Integrations and enable relevant AWS services like GuardDuty, Inspector, Macie, etc. "
                            "Alternatively, use the AWS CLI command: "
                            f"aws securityhub enable-import-findings-for-product --product-arn [PRODUCT_ARN] --region {region}"
                        )
                    )
                )
            else:
                # Security Hub is enabled with products
                product_names = []
                for product_arn in enabled_products:
                    # Extract the product name from the ARN
                    if '/product-subscription/' in product_arn:
                        product_name = product_arn.split('/product-subscription/')[1]
                        product_names.append(product_name)
                    else:
                        product_names.append(product_arn)
                
                products_list = ', '.join(product_names) if product_names else "None"
                
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="List of products that are enabled in Security Hub",
                        actual_value=f"Security Hub has enabled security integrations in region {region}: {products_list}",
                        remediation="No remediation needed"
                    )
                )
        
        return findings



"""
SRA-CLOUDTRAIL-01: Organization CloudTrail Configuration.
"""
from typing import List, Dict, Any
from sraverify.services.cloudtrail.base import CloudTrailCheck
from sraverify.core.logging import logger


class SRA_CLOUDTRAIL_01(CloudTrailCheck):
    """Check if an Organization trail is configured for the AWS Organization."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CLOUDTRAIL-01"
        self.check_name = "An Organization trail is configured for the AWS Organization"
        self.account_type = "management"
        self.severity = "HIGH"
        self.description = (
            "This check verifies that an organization trail is configured for your AWS Organization. "
            "It is important to have uniform logging strategy for your AWS environment. Organization trail "
            "logs all events for all AWS accounts in that organization and delivers logs to a single S3 bucket, "
            "CloudWatch Logs and Event Bridge. Organization trails are automatically applied to all member accounts "
            "in the organization. Member accounts can see the organization trail, but can't modify or delete it. "
            "Organization trail should be configured for all AWS regions even if you are not operating out of any region."
        )
        self.check_logic = (
            "Check if at least one trail has IsOrganizationTrail set to true."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Get all trails using the base class method
        # This will use the cache if available or make API calls if needed
        all_trails = self.describe_trails()
        
        # Filter for organization trails
        org_trails = [
            trail for trail in all_trails 
            if trail.get('IsOrganizationTrail', False)
        ]
        
        if not org_trails:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=f"organization/{self.account_id}",
                    checked_value="IsOrganizationTrail: true",
                    actual_value="No organization trails found",
                    remediation=(
                        "Create an organization trail in the management account using the AWS CLI command: "
                        f"aws cloudtrail create-trail --name org-trail --is-organization-trail --s3-bucket-name cloudtrail-logs-{self.account_id} "
                        f"--is-multi-region-trail --region {self.regions[0] if self.regions else 'us-east-1'}"
                    )
                )
            )
            return findings
        
        # If we have organization trails, create a PASS finding for each one
        for trail in org_trails:
            trail_name = trail.get('Name', 'Unknown')
            trail_arn = trail.get('TrailARN', 'Unknown')
            
            findings.append(
                self.create_finding(
                    status="PASS",
                    region="global",
                    resource_id=trail_arn,
                    checked_value="IsOrganizationTrail: true",
                    actual_value=f"Organization trail '{trail_name}' is configured",
                    remediation="No remediation needed"
                )
            )
        
        return findings



from typing import Dict, List, Any
from sraverify.services.waf.base import WAFCheck

class SRA_WAF_02(WAFCheck):
    def __init__(self):
        super().__init__()
        self.check_id = "SRA-WAF-02"
        self.check_name = "Application Load Balancers should be associated with AWS WAF"
        self.description = "Ensures that all Application Load Balancers are protected by AWS WAF web ACLs to filter malicious traffic"
        self.severity = "HIGH"
        self.check_logic = "Lists all Application Load Balancers and verifies each has a WAF web ACL associated"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            load_balancers_response = self.get_load_balancers(region)

            if "Error" in load_balancers_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=load_balancers_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for ELB and WAF API access"
                ))
                continue

            load_balancers = load_balancers_response.get("LoadBalancers", [])
            
            # Filter for Application Load Balancers only
            albs = [lb for lb in load_balancers if lb.get("Type") == "application"]

            if not albs:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="No ALBs",
                    actual_value="No Application Load Balancers found",
                    remediation="No action needed"
                ))
                continue

            for alb in albs:
                alb_arn = alb.get("LoadBalancerArn")
                alb_name = alb.get("LoadBalancerName")
                
                client = self.get_client(region)
                if not client:
                    continue

                web_acl_response = client.get_web_acl_for_resource(alb_arn)

                if "Error" in web_acl_response:
                    self.findings.append(self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=alb_name,
                        actual_value=web_acl_response["Error"].get("Message", "Unknown error"),
                        remediation="Check IAM permissions for WAF API access"
                    ))
                    continue

                web_acl = web_acl_response.get("WebACL")

                if web_acl:
                    web_acl_name = web_acl.get("Name", "Unknown")
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=alb_name,
                        actual_value=f"WAF Web ACL associated: {web_acl_name}",
                        remediation="No action needed"
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=alb_name,
                        actual_value="No WAF Web ACL associated",
                        remediation="Associate a WAF Web ACL with this Application Load Balancer using the AWS Console, CLI, or API"
                    ))

        return self.findings



"""
Base class for IAM Access Analyzer security checks.
"""
from typing import List, Optional, Dict, Any
import boto3
from sraverify.core.check import SecurityCheck
from sraverify.services.accessanalyzer.client import AccessAnalyzerClient
from sraverify.core.logging import logger


class AccessAnalyzerCheck(SecurityCheck):
    """Base class for all IAM Access Analyzer security checks."""
    
    # Class-level caches shared across all instances
    _delegated_admin_cache = {}
    _analyzer_cache = {}
    
    def __init__(self):
        """Initialize IAM Access Analyzer base check."""
        super().__init__(
            account_type="application",
            service="IAM Access Analyzer",
            resource_type="AWS::AccessAnalyzer::Analyzer"
        )
    
    def _setup_clients(self):
        """Set up AccessAnalyzer clients for enabled regions."""
        # Clear existing clients
        self._clients.clear()
        
        if not hasattr(self, 'session') or not self.session:
            logger.debug("No session available, skipping client setup")
            return
            
        # For organization checks, we need to check all specified regions
        for region in self.regions:
            try:
                client = AccessAnalyzerClient(region, self.session)
                if client.is_access_analyzer_available():
                    self._clients[region] = client
                    logger.debug(f"Access Analyzer client set up for region {region}")
                else:
                    logger.debug(f"Access Analyzer not available in region {region}")
            except Exception as e:
                # Skip regions where client creation fails
                logger.warning(f"Failed to create Access Analyzer client for region {region}: {e}")
                continue
    
    def get_client(self, region: str) -> Optional[AccessAnalyzerClient]:
        """
        Get Access Analyzer client for a region.
        
        Args:
            region: AWS region name
            
        Returns:
            AccessAnalyzerClient for the region or None if not available
        """
        return self._clients.get(region)
    
    def get_analyzers(self, region: str) -> List[Dict[str, Any]]:
        """
        Get analyzers for a specific region with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            List of analyzers in the region
        """
        # Check class-level cache
        cache_key = f"{self.session.region_name}:{region}"
        if cache_key in AccessAnalyzerCheck._analyzer_cache:
            logger.debug(f"Using cached analyzers for {region}")
            return AccessAnalyzerCheck._analyzer_cache[cache_key]
        
        # Get client
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Access Analyzer client available for region {region}")
            return []
        
        # Get analyzers
        logger.debug(f"Fetching analyzers for {region}")
        analyzers = client.list_analyzers()
        
        # Cache the analyzers
        AccessAnalyzerCheck._analyzer_cache[cache_key] = analyzers
        logger.debug(f"Cached {len(analyzers)} analyzers for {region}")
        
        return analyzers
        
    def get_delegated_admin(self) -> Dict[str, Any]:
        """
        Get the delegated administrator for IAM Access Analyzer with caching.
        
        Returns:
            Dictionary containing delegated administrator details or empty dict if none
        """
        account_id = self.account_id
        
        # Check class-level cache
        if account_id in AccessAnalyzerCheck._delegated_admin_cache:
            logger.debug(f"Using cached delegated admin for account {account_id}")
            return AccessAnalyzerCheck._delegated_admin_cache[account_id]
        
        # If not in cache, get it from the client
        # Use the first available region to make the API call
        if not self._clients:
            logger.warning("No Access Analyzer clients available")
            return {}
        
        # Use the first available region's client
        region = next(iter(self._clients))
        client = self._clients[region]
        
        # Get delegated admin from client
        delegated_admin = client.get_delegated_admin()
        
        # Cache the result
        AccessAnalyzerCheck._delegated_admin_cache[account_id] = delegated_admin
        
        return delegated_admin



"""
SRA-MACIE-04: Checks that findings are being exported to S3 in the log archive account are encrypted at rest.
"""
from typing import List, Dict, Any
from sraverify.services.macie.base import MacieCheck
from sraverify.core.logging import logger


class SRA_MACIE_04(MacieCheck):
    """Check if Macie findings exported to S3 are encrypted at rest using KMS."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-MACIE-04"
        self.check_name = "Checks that findings are being exported to S3 in the log archive account are encrypted at rest"
        self.description = (
            "This check verifies whether all Macie findings that are being exported to a S3 bucket within the Log Archive account "
            "are encrypted using KMS key. Macie findings are sensitive in natures and should be encrypted to prevent from unauthorized disclosure."
        )
        self.severity = "HIGH"
        self.account_type = "application"
        self.check_logic = "Check validates using get-classification-export-configuration that kms key exists. PASS if KMS ARN returned."
        self.resource_type = "AWS::Macie::Session"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        for region in self.regions:
            # Get classification export configuration using the base class method with caching
            export_config = self.get_classification_export_configuration(region)
            
            # Check if the API call was successful
            if not export_config:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="KMS encryption for S3 bucket",
                        actual_value="Failed to retrieve Macie classification export configuration",
                        remediation="Ensure Macie is enabled and you have the necessary permissions to call the Macie GetClassificationExportConfiguration API"
                    )
                )
                continue
            
            # Check if export configuration exists
            configuration = export_config.get('configuration', {})
            if not configuration:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="KMS encryption for S3 bucket",
                        actual_value="Macie findings export configuration not found",
                        remediation=(
                            f"Configure Macie to export findings to a S3 bucket with KMS encryption in region {region} using the AWS CLI command: "
                            f"aws macie2 put-classification-export-configuration --s3-destination bucketName=your-bucket-name,kmsKeyArn=arn:aws:kms:{region}:{self.account_id}:key/your-key-id --region {region}"
                        )
                    )
                )
                continue
            
            # Check if S3 destination exists
            s3_destination = configuration.get('s3Destination', {})
            if not s3_destination:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="KMS encryption for S3 bucket",
                        actual_value="S3 destination not found in Macie findings export configuration",
                        remediation=(
                            f"Configure Macie to export findings to a S3 bucket with KMS encryption in region {region} using the AWS CLI command: "
                            f"aws macie2 put-classification-export-configuration --s3-destination bucketName=your-bucket-name,kmsKeyArn=arn:aws:kms:{region}:{self.account_id}:key/your-key-id --region {region}"
                        )
                    )
                )
                continue
            
            # Get bucket name and KMS key ARN
            bucket_name = s3_destination.get('bucketName', '')
            kms_key_arn = s3_destination.get('kmsKeyArn', '')
            
            # Check if KMS key ARN exists
            if kms_key_arn:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="KMS encryption for S3 bucket",
                        actual_value=f"Macie findings exported to S3 bucket '{bucket_name}' are encrypted using KMS key '{kms_key_arn}' in region {region}",
                        remediation="No remediation needed"
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"macie2/{self.account_id}/{region}",
                        checked_value="KMS encryption for S3 bucket",
                        actual_value=f"Macie findings exported to S3 bucket '{bucket_name}' are not encrypted using KMS in region {region}",
                        remediation=(
                            f"Configure Macie to export findings to a S3 bucket with KMS encryption in region {region} using the AWS CLI command: "
                            f"aws macie2 put-classification-export-configuration --s3-destination bucketName={bucket_name},kmsKeyArn=arn:aws:kms:{region}:{self.account_id}:key/your-key-id --region {region}"
                        )
                    )
                )
        
        return findings



"""
Progress tracking for scan operations.
"""
import time
from typing import Optional
from colorama import Fore, Style

class ScanProgress:
    """Progress tracker for scan operations."""
    
    SPINNER = ['/', '-', '\\', '|']
    
    def __init__(self, total_checks: int):
        """
        Initialize the progress tracker.
        
        Args:
            total_checks: The total number of checks to be performed
        
        Raises:
            ValueError: If total_checks is less than or equal to 0
        """
        if total_checks <= 0:
            raise ValueError("total_checks must be greater than 0")
        self.total_checks = total_checks
        self.completed_checks = 0
        self.start_time = time.time()
        self.current_service: Optional[str] = None
        self.spinner_index = 0
        self.last_print_time = 0
        self.print_interval = 0.1  # Update display every 0.1 seconds
    
    @property
    def progress(self) -> float:
        """
        Calculate the progress percentage.
        
        Returns:
            Progress percentage
        """
        return (self.completed_checks / self.total_checks * 100)
    
    @property
    def duration(self) -> str:
        """
        Calculate the elapsed time in minutes:seconds format.
        
        Returns:
            Elapsed time string
        """
        elapsed = int(time.time() - self.start_time)
        minutes = elapsed // 60
        seconds = elapsed % 60
        return f"{minutes}:{seconds:02d}"
    
    def update(self, service: str):
        """
        Update the current service being scanned.
        
        Args:
            service: Service name
        """
        self.current_service = service
        self.print_progress()
        
    def increment(self):
        """Increment the completed checks counter."""
        self.completed_checks += 1
        self.spinner_index = (self.spinner_index + 1) % len(self.SPINNER)
        self.print_progress()
        
    def print_progress(self):
        """Print the current progress status with rate limiting."""
        current_time = time.time()
        # Only update display if enough time has passed since last update
        if current_time - self.last_print_time >= self.print_interval:
            self._do_print()
            self.last_print_time = current_time

    def _do_print(self):
        """Actually perform the progress printing."""
        spinner = self.SPINNER[self.spinner_index]
        
        # Calculate the width of the progress bar (50 characters)
        bar_width = 50
        filled_length = int(self.progress / 100 * bar_width)
        bar = '=' * filled_length + ' ' * (bar_width - filled_length)
        
        # Format the progress bar with current status
        print(f"{Fore.BLUE}\r-> Scanning {self.current_service} service {Style.RESET_ALL} |{bar}| {spinner} "
              f"{self.completed_checks}/{self.total_checks} "
              f"[{self.progress:.0f}%] in {self.duration}", 
              end="", flush=True)
    
    def finish(self):
        """Complete the progress display with a newline."""
        print()  # Print newline to finish the progress display



"""Check if Amazon Security Lake is enabled."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_01(SecurityLakeCheck):
    """Check if Amazon Security Lake is enabled."""

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.account_type = "log-archive"  # Check all org accounts from delegated admin
        self.check_id = "SRA-SECURITYLAKE-01"
        self.check_name = "Security Lake is enabled for all organization accounts"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether Amazon Security Lake is enabled for all active accounts in the organization. "
            "Amazon Security Lake is a fully managed security data lake service that you "
            "can use to automatically centralize security data from AWS environments, "
            "SaaS providers, on premises, cloud sources, and third-party sources into a "
            "purpose-built data lake that's stored in your AWS account. The data lake is "
            "backed by Amazon S3 buckets, and you retain ownership over your data. You "
            "must enable security lake in every AWS account and AWS region to collect "
            "security logs and event from your entire AWS environment. "
            "This check runs from the delegated administrator account "
            "and validates configuration across all organization member accounts."
        )
        self.check_logic = (
            "Checks if Security Lake is enabled for all active organization accounts by calling get_data_lake_sources API. "
            "The check passes if Security Lake is enabled for all active accounts in the organization. "
            "The check fails if any active account does not have Security Lake enabled."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """

        for region in self.regions:
            logger.debug(f"Checking if Security Lake is enabled for all organization accounts in {region}")

            # Get all organization accounts
            org_accounts = self.get_organization_accounts(region)
            if not org_accounts:
                logger.debug("No organization accounts found, checking current account only")
                org_accounts = [{'Id': self.account_id, 'Status': 'ACTIVE'}]

            # Create sets of active account IDs
            active_org_account_ids = set()
            for account in org_accounts:
                if account.get('Status') == 'ACTIVE':
                    active_org_account_ids.add(account.get('Id'))

            # Get accounts with Security Lake enabled
            enabled_accounts = set()
            sources_data = self.get_data_lake_sources(region)  # No account_id = get all accounts
            for source in sources_data:
                account_id = source.get('account')
                if account_id:
                    enabled_accounts.add(account_id)

            # Check each account in the organization
            for account_id in active_org_account_ids:
                resource_id = f"arn:aws:securitylake:{region}:{account_id}:datalake/default"

                if account_id not in enabled_accounts:
                    self.findings.append(
                        self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=resource_id,
                            checked_value="Security Lake enabled",
                            actual_value=f"Security Lake is not enabled for account {account_id}",
                            remediation=(
                                f"Enable Security Lake for account {account_id}. In the Security Lake console, "
                                "navigate to Settings and enable Security Lake. Alternatively, use the AWS CLI command: "
                                f"aws securitylake create-data-lake --region {region}"
                            )
                        )
                    )
                else:
                    self.findings.append(
                        self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=resource_id,
                            checked_value="Security Lake enabled",
                            actual_value=f"Security Lake is enabled for account {account_id}",
                            remediation="No remediation needed"
                        )
                    )

        return self.findings



from typing import Dict, List, Any
from sraverify.services.firewallmanager.base import FirewallManagerCheck

class SRA_FIREWALLMANAGER_06(FirewallManagerCheck):
    def __init__(self):
        super().__init__()
        self.check_id = "SRA-FIREWALLMANAGER-06"
        self.check_name = "Firewall Manager manages AWS Network Firewall policies"
        self.description = "Verifies that AWS Firewall Manager has Network Firewall policies configured in each region"
        self.severity = "MEDIUM"
        self.check_logic = "Calls list_policies() per region and checks for policies with SecurityServiceType of NETWORK_FIREWALL"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            policies_response = self.list_policies(region)
            
            if "Error" in policies_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=policies_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Firewall Manager API access"
                ))
                continue
            
            policies = policies_response.get("PolicyList", [])
            nfw_policies = [
                p for p in policies 
                if p.get("SecurityServiceType") == "NETWORK_FIREWALL"
            ]
            
            if not nfw_policies:
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="No Network Firewall policies configured",
                    remediation="Create Firewall Manager Network Firewall policies: https://docs.aws.amazon.com/waf/latest/developerguide/network-firewall-policies.html"
                ))
            else:
                policy_names = [p.get("PolicyName", "Unknown") for p in nfw_policies]
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id=",".join([p.get("PolicyId", "") for p in nfw_policies]),
                    actual_value=f"{len(nfw_policies)} Network Firewall policy(ies) configured: {', '.join(policy_names)}",
                    remediation="No remediation needed"
                ))
        
        return self.findings



from typing import Dict, List, Any
from sraverify.services.firewallmanager.base import FirewallManagerCheck

class SRA_FIREWALLMANAGER_09(FirewallManagerCheck):
    def __init__(self):
        super().__init__()
        self.check_id = "SRA-FIREWALLMANAGER-09"
        self.check_name = "Firewall Manager policies are in active status"
        self.description = "Verifies that AWS Firewall Manager policies are in ACTIVE status and not out of admin scope"
        self.severity = "HIGH"
        self.check_logic = "Calls list_policies() per region and checks that all policies have PolicyStatus set to ACTIVE"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            policies_response = self.list_policies(region)
            
            if "Error" in policies_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=policies_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Firewall Manager API access"
                ))
                continue
            
            policies = policies_response.get("PolicyList", [])
            
            if not policies:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id=None,
                    actual_value="No Firewall Manager policies configured in region",
                    remediation="No remediation needed"
                ))
                continue
            
            for policy in policies:
                policy_id = policy.get("PolicyId", "")
                policy_name = policy.get("PolicyName", "Unknown")
                policy_status = policy.get("PolicyStatus", "")
                
                if policy_status != "ACTIVE":
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=policy_id,
                        actual_value=f"Policy '{policy_name}' has status '{policy_status}'",
                        remediation=f"Ensure policy '{policy_name}' is within admin scope to make it ACTIVE"
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=policy_id,
                        actual_value=f"Policy '{policy_name}' is ACTIVE",
                        remediation="No remediation needed"
                    ))
        
        return self.findings



"""Check if Security Lake organization auto-enable configuration matches AWS defaults."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_05(SecurityLakeCheck):
    """Check if Security Lake organization auto-enable configuration matches AWS defaults."""

    # AWS default/recommended log sources for new accounts
    AWS_DEFAULT_LOG_SOURCES = {
        "CLOUD_TRAIL_MGMT",
        "LAMBDA_EXECUTION", 
        "EKS_AUDIT",
        "ROUTE53",
        "SH_FINDINGS",
        "VPC_FLOW"
    }

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.account_type = "log-archive"  # Organization config managed by delegated admin
        self.check_id = "SRA-SECURITYLAKE-05"
        self.check_name = "Security Lake organization auto-enable matches AWS defaults"
        self.severity = "MEDIUM"
        self.description = (
            "This check verifies whether Amazon Security Lake organization auto-enable "
            "configuration matches AWS default/recommended log sources for new accounts "
            "(CLOUD_TRAIL_MGMT, LAMBDA_EXECUTION, EKS_AUDIT, ROUTE53, SH_FINDINGS, VPC_FLOW). "
            "S3_DATA and WAF are excluded as they are optional due to high volume."
        )
        self.check_logic = (
            "Gets the organization configuration for Security Lake auto-enable settings. "
            "The check passes if all AWS default log sources are configured for auto-enable. "
            "The check fails if any default log sources are missing from auto-enable configuration."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """

        for region in self.regions:
            logger.debug(f"Checking Security Lake organization auto-enable configuration in {region}")
            resource_id = f"arn:aws:securitylake:{region}:{self.account_id}:organization-configuration/auto-enable"

            # Get organization configuration using the base class method
            config = self.get_organization_configuration(region)

            if not config:
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Auto-enable configured with AWS defaults: {', '.join(sorted(self.AWS_DEFAULT_LOG_SOURCES))}",
                        actual_value=f"No organization configuration found in {region}",
                        remediation=(
                            "Configure Security Lake organization auto-enable settings. In the Security Lake console, "
                            "navigate to Settings > Organization Configuration and enable auto-enable for new accounts "
                            "with the AWS default log sources."
                        )
                    )
                )
                continue

            # Extract auto-enable sources from configuration
            auto_enable_sources = set()
            auto_enable_config = config.get("autoEnableNewAccount", [])
            
            # Find the configuration for this region
            for region_config in auto_enable_config:
                if region_config.get("region") == region:
                    sources = region_config.get("sources", [])
                    for source in sources:
                        source_name = source.get("sourceName")
                        if source_name:
                            auto_enable_sources.add(source_name)
                    break

            missing_sources = self.AWS_DEFAULT_LOG_SOURCES - auto_enable_sources

            if missing_sources:
                actual_configured = ', '.join(sorted(auto_enable_sources)) if auto_enable_sources else "None"
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Auto-enable configured with AWS defaults: {', '.join(sorted(self.AWS_DEFAULT_LOG_SOURCES))}",
                        actual_value=f"Currently configured: {actual_configured}. Missing: {', '.join(sorted(missing_sources))}",
                        remediation=(
                            "Update Security Lake organization auto-enable configuration to include missing sources. "
                            "In the Security Lake console, navigate to Settings > Organization Configuration and "
                            f"add the missing sources: {', '.join(sorted(missing_sources))}."
                        )
                    )
                )
            else:
                self.findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Auto-enable configured with AWS defaults: {', '.join(sorted(self.AWS_DEFAULT_LOG_SOURCES))}",
                        actual_value=f"Currently configured: {', '.join(sorted(auto_enable_sources))}",
                        remediation="No remediation needed"
                    )
                )

        return self.findings



"""
S3 security checks.
"""
from sraverify.services.s3.checks.sra_s3_01 import SRA_S3_01
from sraverify.services.s3.checks.sra_s3_02 import SRA_S3_02
from sraverify.services.s3.checks.sra_s3_03 import SRA_S3_03
from sraverify.services.s3.checks.sra_s3_04 import SRA_S3_04

# Register checks
CHECKS = {
    "SRA-S3-01": SRA_S3_01,
    "SRA-S3-02": SRA_S3_02,
    "SRA-S3-03": SRA_S3_03,
    "SRA-S3-04": SRA_S3_04,

}



"""
SRA-CONFIG-01: AWS Config Recorder Configured.
"""
from typing import List, Dict, Any
from sraverify.services.config.base import ConfigCheck
from sraverify.core.logging import logger


class SRA_CONFIG_01(ConfigCheck):
    """Check if AWS Config recorder is configured in each region."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CONFIG-01"
        self.check_name = "AWS Config recorder is configured in this region"
        self.account_type = "application"  
        self.severity = "HIGH"
        self.description = (
            "This check verifies that a configuration recorder exists in the AWS Region. "
            "AWS Config uses the configuration recorder to detect changes in your resource configurations "
            "and capture these changes as configuration items. You must create a configuration recorder "
            "in every AWS Region for AWS Config can track your resource configurations in the region."
        )
        self.check_logic = (
            "Checks if AWS Config recorder exists in each region using describe-configuration-recorder-status API."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        if not self.regions:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id="config:global",
                    actual_value="No regions specified for check",
                    remediation="Specify at least one region when running the check"
                )
            )
            return findings
        
        # Check each region for configuration recorder
        for region in self.regions:
            # Get configuration recorders for the region
            recorders = self.get_configuration_recorders(region)
            
            # Get configuration recorder status for the region
            recorder_statuses = self.get_configuration_recorder_status(region)
            
            if not recorders:
                # No configuration recorder found in this region
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"arn:aws:config:{region}:{self.account_id}:configurationRecorder/default",
                        actual_value="No configuration recorder found in this region",
                        remediation=(
                            f"1. Check if the AWS Config service-linked role exists: aws iam get-role --role-name AWSServiceRoleForConfig. "
                            f"2. If the role doesn't exist, create it: aws iam create-service-linked-role --aws-service-name config.amazonaws.com. "
                            f"3. Create a configuration recorder in {region}: aws configservice put-configuration-recorder --configuration-recorder name=default,roleARN=arn:aws:iam::{self.account_id}:role/aws-service-role/config.amazonaws.com/AWSServiceRoleForConfig --recording-group allSupported=true,includeGlobalResourceTypes=true --region {region}"
                        )
                    )
                )
            else:
                # Configuration recorder exists, check if it's enabled
                recorder_name = recorders[0].get('name', 'default')
                recorder_role_arn = recorders[0].get('roleARN', '')
                
                # Construct the Config Recorder ARN
                recorder_arn = f"arn:aws:config:{region}:{self.account_id}:configurationRecorder/{recorder_name}"
                
                # Find the status for this recorder
                recorder_status = next((status for status in recorder_statuses if status.get('name') == recorder_name), None)
                
                if recorder_status and recorder_status.get('recording', False):
                    # Configuration recorder exists and is recording
                    findings.append(
                        self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=recorder_arn,
                            actual_value=f"Configuration recorder '{recorder_name}' exists and is recording",
                            remediation="No remediation needed"
                        )
                    )
                elif recorder_status:
                    # Configuration recorder exists but is not recording
                    findings.append(
                        self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=recorder_arn,
                            actual_value=f"Configuration recorder '{recorder_name}' exists but is not recording",
                            remediation=(
                                f"Start the configuration recorder in {region} using the AWS CLI command: "
                                f"aws configservice start-configuration-recorder --configuration-recorder-name {recorder_name} --region {region}"
                            )
                        )
                    )
                else:
                    # Configuration recorder exists but no status found
                    findings.append(
                        self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=recorder_arn,
                            actual_value=f"Configuration recorder '{recorder_name}' exists but status could not be determined",
                            remediation=(
                                f"Check the configuration recorder status in {region} and ensure it's properly configured"
                            )
                        )
                    )
        
        return findings



"""
Check if Shield engagement Lambda function is configured.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_11(ShieldCheck):
    """Check if Shield engagement Lambda function is configured."""

    def __init__(self):
        """Initialize Shield engagement Lambda function check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-11"
        self.check_name = "Shield engagement Lambda function is configured"
        self.description = ("This check verifies that a Lambda function named "
                            "'AWS_Shield_Engagement_Lambda' exists to automate support case creation during DDoS events.")
        self.severity = "MEDIUM"
        self.check_logic = "Check for existence of Lambda function named 'AWS_Shield_Engagement_Lambda' in us-east-1."

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Check for Lambda function in us-east-1
        region = "us-east-1"
        function_name = "AWS_Shield_Engagement_Lambda"

        lambda_function = self.get_lambda_function(region, function_name)

        if "Error" in lambda_function:
            error_code = lambda_function["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield engagement Lambda function not found",
                    remediation=f"Create Lambda function named '{function_name}' to automate support case creation during DDoS events"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=lambda_function["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Lambda API access"
                ))
        elif lambda_function.get("Configuration"):
            function_arn = lambda_function["Configuration"].get("FunctionArn", "")
            runtime = lambda_function["Configuration"].get("Runtime", "")
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id=function_arn,
                actual_value=f"Shield engagement Lambda function exists (Runtime: {runtime})",
                remediation=""
            ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="Shield engagement Lambda function not found",
                remediation=f"Create Lambda function named '{function_name}' to automate support case creation during DDoS events"
            ))

        return self.findings



"""
Base class for GuardDuty security checks.
"""
from typing import List, Optional, Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.guardduty.client import GuardDutyClient
from sraverify.core.logging import logger


class GuardDutyCheck(SecurityCheck):
    """Base class for all GuardDuty security checks."""
    
    # Class-level caches shared across all instances
    _detector_details_cache = {}
    _detector_ids_cache = {}
    _org_config_cache = {}
    _admin_accounts_cache = {}
    
    def __init__(self):
        """Initialize GuardDuty base check."""
        super().__init__(
            account_type="application",
            service="GuardDuty",
            resource_type="AWS::GuardDuty::Detector"
        )
    
    def _setup_clients(self):
        """Set up GuardDuty clients for each region."""
        # Clear existing clients
        self._clients.clear()
        # Set up new clients only if regions are initialized
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = GuardDutyClient(region, session=self.session)
    
    def get_detector_id(self, region: str) -> Optional[str]:
        """
        Get detector ID for a specific region with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Detector ID if available, None otherwise
        """
        # Check class-level cache
        cache_key = f"{self.session.region_name}:{region}"
        if cache_key in GuardDutyCheck._detector_ids_cache:
            logger.debug(f"GuardDuty: Using cached detector ID for {region}")
            return GuardDutyCheck._detector_ids_cache[cache_key]
        
        # Get client
        client = self.get_client(region)
        if not client:
            logger.warning(f"GuardDuty: No GuardDuty client available for region {region}")
            return None
        
        # Get detector ID
        logger.debug(f"GuardDuty: Fetching detector ID for {region}")
        detector_id = client.get_detector_id()
        
        # Check if detector_id contains an error
        if detector_id and isinstance(detector_id, str) and detector_id.startswith("ERROR:"):
            _, error_code, error_message = detector_id.split(":", 2)
            logger.warning(f"GuardDuty: Error accessing GuardDuty in {region}: {error_code}")
            GuardDutyCheck._detector_ids_cache[cache_key] = None
            return None
        
        # Cache the detector ID
        if detector_id:
            logger.debug(f"GuardDuty: Found detector ID {detector_id} for {region}")
            GuardDutyCheck._detector_ids_cache[cache_key] = detector_id
        else:
            logger.debug(f"GuardDuty: No detector ID found for {region}")
        
        return detector_id
    
    def get_detector_details(self, region: str) -> Dict[str, Any]:
        """
        Get detector details for a specific region.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing detector details or empty dict if not available
        """
        # Check if we already have cached details in the class-level cache
        cache_key = f"{self.session.region_name}:{region}"  # Include session region to avoid conflicts
        if cache_key in GuardDutyCheck._detector_details_cache:
            logger.debug(f"GuardDuty: Using cached detector details for {region}")
            return GuardDutyCheck._detector_details_cache[cache_key]
        
        # Get detector ID
        detector_id = self.get_detector_id(region)
        if not detector_id:
            logger.debug(f"GuardDuty: No detector ID found for region {region}")
            return {}
        
        # Get client
        client = self.get_client(region)
        if not client:
            logger.warning(f"GuardDuty: No GuardDuty client available for region {region}")
            return {}
        
        # Get detector details
        logger.debug(f"GuardDuty: Getting detector details for {detector_id} in {region}")
        details = client.get_detector_details(detector_id)
        
        # Cache the details in the class-level cache
        GuardDutyCheck._detector_details_cache[cache_key] = details
        logger.debug(f"GuardDuty: Cached detector details for {region}")
        
        return details
    
    def get_organization_configuration(self, region: str) -> Dict[str, Any]:
        """
        Get organization configuration for a specific region.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing organization configuration details or empty dict if not available
        """
        # Check if we already have cached org config in the class-level cache
        cache_key = f"{self.session.region_name}:{region}"
        if cache_key in GuardDutyCheck._org_config_cache:
            logger.debug(f"GuardDuty: Using cached organization configuration for {region}")
            return GuardDutyCheck._org_config_cache[cache_key]
        
        # Get detector ID
        detector_id = self.get_detector_id(region)
        if not detector_id:
            logger.debug(f"GuardDuty: No detector ID found for region {region}")
            return {}
        
        # Get client
        client = self.get_client(region)
        if not client:
            logger.warning(f"GuardDuty: No GuardDuty client available for region {region}")
            return {}
        
        # Get organization configuration
        logger.debug(f"GuardDuty: Getting organization configuration for {detector_id} in {region}")
        org_config = client.describe_organization_configuration(detector_id)
        
        # Cache the org config in the class-level cache
        GuardDutyCheck._org_config_cache[cache_key] = org_config
        logger.debug(f"GuardDuty: Cached organization configuration for {region}")
        
        return org_config
    
    def list_organization_admin_accounts(self, region: str) -> Dict[str, Any]:
        """
        List organization admin accounts for GuardDuty.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing organization admin accounts details or empty dict if not available
        """
        # Check if we already have cached admin accounts in the class-level cache
        cache_key = f"{self.session.region_name}:{region}"
        if cache_key in GuardDutyCheck._admin_accounts_cache:
            logger.debug(f"GuardDuty: Using cached organization admin accounts for {region}")
            return GuardDutyCheck._admin_accounts_cache[cache_key]
        
        # Get client
        client = self.get_client(region)
        if not client:
            logger.warning(f"GuardDuty: No GuardDuty client available for region {region}")
            return {}
        
        # List organization admin accounts
        logger.debug(f"GuardDuty: Listing organization admin accounts in {region}")
        admin_accounts = client.list_organization_admin_accounts()
        
        # Cache the admin accounts in the class-level cache
        GuardDutyCheck._admin_accounts_cache[cache_key] = admin_accounts
        logger.debug(f"GuardDuty: Cached organization admin accounts for {region}")
        
        return admin_accounts
    
    def get_enabled_regions(self) -> List[str]:
        """
        Get list of regions where GuardDuty is enabled.
        
        Returns:
            List of region names where GuardDuty is enabled
        """
        # If no detector IDs have been discovered yet, try to discover them
        if not any(k.endswith(f":{region}") for k in GuardDutyCheck._detector_ids_cache.keys() 
                  for region in self.regions):
            logger.debug("GuardDuty: No detector IDs cached, discovering them now")
            for region in self.regions:
                self.get_detector_id(region)
        
        # Get regions from the class-level cache that match the current session
        prefix = f"{self.session.region_name}:"
        enabled_regions = [
            key.split(":")[-1] for key in GuardDutyCheck._detector_ids_cache.keys()
            if key.startswith(prefix) and GuardDutyCheck._detector_ids_cache[key] is not None
        ]
        
        return enabled_regions



"""
Check if Shield Advanced is configured for Elastic IP addresses.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_05(ShieldCheck):
    """Check if Shield Advanced is configured for Elastic IP addresses."""

    def __init__(self):
        """Initialize Shield Advanced Elastic IP protection check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-05"
        self.check_name = "Shield Advanced is configured for Elastic IP addresses"
        self.description = ("This check verifies that AWS Shield Advanced is protecting "
                            "at least one Elastic IP address.")
        self.severity = "HIGH"
        self.check_logic = ("List Shield protections and filter by Elastic IP ARNs. "
                            "Check fails if no Elastic IP addresses are protected.")

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        protections = self.list_protections(region)

        if "Error" in protections:
            error_code = protections["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield Advanced subscription not found",
                    remediation="Enable Shield Advanced subscription to protect resources"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=protections["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif protections.get("Protections"):
            # Filter for Elastic IPs by checking ResourceArn
            eip_protections = [
                p for p in protections["Protections"]
                if "eip-allocation" in p.get("ResourceArn", "").lower()
            ]

            if eip_protections:
                protected_count = len(eip_protections)
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="shield:eip-protections",
                    actual_value=f"{protected_count} Elastic IP address(es) protected",
                    remediation=""
                ))
            else:
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="No Elastic IP addresses protected",
                    remediation="Enable Shield Advanced protection for Elastic IP addresses in the AWS Shield console"
                ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No Elastic IP addresses protected",
                remediation="Enable Shield Advanced protection for Elastic IP addresses in the AWS Shield console"
            ))

        return self.findings



"""
Base class for EC2 security checks.
"""
from typing import Dict, Optional, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.ec2.client import EC2Client
from sraverify.core.logging import logger


class EC2Check(SecurityCheck):
    """Base class for all EC2 security checks."""
    
    # Class-level caches shared across all instances
    _ebs_encryption_default_cache = {}
    
    def __init__(self):
        """Initialize EC2 base check."""
        super().__init__(
            account_type="application",
            service="EC2",
            resource_type="AWS::EC2::Instance"
        )
    
    def _setup_clients(self):
        """Set up EC2 clients for each region."""
        # Clear existing clients
        self._clients.clear()
        # Set up new clients only if regions are initialized
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = EC2Client(region, session=self.session)
    
    def get_client(self, region: str) -> Optional[EC2Client]:
        """
        Get EC2 client for a specific region.
        
        Args:
            region: AWS region name
            
        Returns:
            EC2Client for the region or None if not available
        """
        return self._clients.get(region)
    
    def get_ebs_encryption_by_default(self, region: str) -> Dict[str, Any]:
        """
        Get the EBS encryption by default status for the account in the region with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing EBS encryption by default status
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._ebs_encryption_default_cache:
            logger.debug(f"Using cached EBS encryption by default status for {region}")
            return self.__class__._ebs_encryption_default_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No EC2 client available for region {region}")
            return {}
        
        # Get EBS encryption by default status from client
        encryption_status = client.get_ebs_encryption_by_default()
        
        # Cache the result
        self.__class__._ebs_encryption_default_cache[cache_key] = encryption_status
        logger.debug(f"Cached EBS encryption by default status for {region}")
        
        return encryption_status



"""
SRA-SECURITYHUB-08: Security Hub check.
"""
from typing import List, Dict, Any, Set
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_08(SecurityHubCheck):
    """Check if all active organization accounts are Security Hub members."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-08"
        self.check_name = "All active organization accounts are Security Hub members"
        self.account_type = "audit"  # This check is for the audit account
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether all active members accounts of the AWS Organization are Security Hub members. "
            "Security Hub provides comprehensive security state and should include all AWS accounts."
        )
        self.check_logic = (
            "Compare the outputs of organizations list-accounts and securityhub list-members. "
            "Make sure that the list includes all accounts, excluding the Security Hub admin (audit account) "
            "which is not considered a member."
        )
        self._audit_accounts = []  # Will be populated from command line args
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check each region separately
        for region in self.regions:
            # Get all organization accounts
            org_accounts = self.get_organization_accounts(region)
            
            # Get Security Hub members
            securityhub_members = self.get_security_hub_members(region)
            
            resource_id = f"securityhub:members/{self.account_id}/{region}"
            
            # Create sets of account IDs for comparison
            active_org_account_ids = set()
            for account in org_accounts:
                if account.get('Status') == 'ACTIVE':
                    active_org_account_ids.add(account.get('Id'))
            
            securityhub_member_ids = set()
            for member in securityhub_members:
                securityhub_member_ids.add(member.get('AccountId'))
            
            # Determine the audit account ID
            audit_account_id = None
            if self._audit_accounts:
                audit_account_id = self._audit_accounts[0]
            else:
                # If no audit account is provided, assume the current account is the audit account
                audit_account_id = self.account_id
            
            # Remove the audit account from the list of active organization accounts
            # since the audit account is the Security Hub admin and not a member
            if audit_account_id in active_org_account_ids:
                active_org_account_ids.remove(audit_account_id)
            
            # Find accounts that should be Security Hub members but aren't
            missing_accounts = active_org_account_ids - securityhub_member_ids
            
            if missing_accounts:
                missing_accounts_list = ', '.join(missing_accounts)
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="All active organization accounts are Security Hub members",
                        actual_value=(
                            f"The following active organization accounts are not Security Hub members in region {region}: "
                            f"{missing_accounts_list}. "
                            f"Active organization accounts: {len(active_org_account_ids)}, "
                            f"Security Hub members: {len(securityhub_member_ids)}"
                        ),
                        remediation=(
                            f"Add the missing accounts as Security Hub members in region {region}. "
                            f"In the AWS Console, navigate to Security Hub in the audit account, go to Settings > Accounts, "
                            f"and add the missing accounts. Alternatively, use the AWS CLI command: "
                            f"aws securityhub create-members --account-details 'AccountId={missing_accounts_list.replace(', ', ',AccountId=')}' --region {region}"
                        )
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="All active organization accounts are Security Hub members",
                        actual_value=(
                            f"All active organization accounts are Security Hub members in region {region}. "
                            f"Active organization accounts (excluding admin): {len(active_org_account_ids)}, "
                            f"Security Hub members: {len(securityhub_member_ids)}"
                        ),
                        remediation="No remediation needed"
                    )
                )
        
        return findings



"""
Account client for interacting with AWS Account Management service.
"""
from typing import Dict, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class AccountClient:
    """Client for interacting with AWS Account Management service."""
    
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        """
        Initialize Account client for a specific region.
        
        Args:
            region: AWS region name
            session: AWS session to use (if None, a new session will be created)
        """
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('account', region_name=region)
    
    def get_alternate_contact(self, contact_type: str, account_id: Optional[str] = None) -> Dict[str, Any]:
        """
        Get alternate contact information for the specified type.
        
        Args:
            contact_type: Type of contact (BILLING, OPERATIONS, or SECURITY)
            account_id: Optional account ID (defaults to current account)
            
        Returns:
            Dictionary containing contact details or error information
        """
        try:
            params = {"AlternateContactType": contact_type}
            if account_id:
                params["AccountId"] = account_id
                
            return self.client.get_alternate_contact(**params)
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = str(e)
            logger.debug(f"Error getting {contact_type} alternate contact in {self.region}: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }



"""
Check if Shield Advanced proactive engagement is enabled.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_09(ShieldCheck):
    """Check if Shield Advanced proactive engagement is enabled."""

    def __init__(self):
        """Initialize Shield Advanced proactive engagement check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-09"
        self.check_name = "Shield Advanced proactive engagement is enabled"
        self.description = ("This check verifies that AWS Shield Advanced proactive engagement "
                            "is enabled, allowing the Shield Response Team to contact you directly during attacks.")
        self.severity = "MEDIUM"
        self.check_logic = ("Get Shield subscription details and check ProactiveEngagementStatus. "
                            "Check fails if proactive engagement is disabled.")

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        subscription = self.get_subscription_state(region)

        if "Error" in subscription:
            error_code = subscription["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield Advanced not subscribed",
                    remediation="Enable Shield Advanced subscription in the AWS Shield console"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=subscription["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif "Subscription" in subscription:
            proactive_status = subscription["Subscription"].get("ProactiveEngagementStatus", "")
            if proactive_status == "ENABLED":
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="shield:proactive-engagement",
                    actual_value="Proactive engagement is enabled",
                    remediation=""
                ))
            elif proactive_status == "PENDING":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id="shield:proactive-engagement",
                    actual_value="Proactive engagement is pending",
                    remediation="Complete proactive engagement setup by providing emergency contacts"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id="shield:proactive-engagement",
                    actual_value=f"Proactive engagement is {proactive_status or 'disabled'}",
                    remediation="Enable proactive engagement using EnableProactiveEngagement API and configure emergency contacts"
                ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="Shield Advanced subscription not found",
                remediation="Enable Shield Advanced subscription in the AWS Shield console"
            ))

        return self.findings



from typing import Dict, List, Any
from sraverify.services.securityincidentresponse.base import SecurityIncidentResponseCheck

class SRA_SECURITYINCIDENTRESPONSE_02(SecurityIncidentResponseCheck):
    def __init__(self):
        super().__init__()
        self.account_type = "audit"
        self.check_id = "SRA-SECURITYINCIDENTRESPONSE-02"
        self.check_name = "Security Incident Response membership active"
        self.description = "Verifies that Security Incident Response membership is active"
        self.severity = "HIGH"
        self.check_logic = "Lists memberships and verifies status is Active"

    def execute(self) -> List[Dict[str, Any]]:
        region = self.regions[0] if self.regions else "us-east-1"
        
        response = self.list_memberships()
        
        if "Error" in response:
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value=response["Error"].get("Message", "Unknown error"),
                remediation="Check IAM permissions for Security Incident Response API access"
            ))
            return self.findings

        memberships = response.get("items", [])
        
        if not memberships:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No Security Incident Response memberships found",
                remediation="Create a Security Incident Response membership through the AWS console or API"
            ))
        else:
            active_found = False
            for membership in memberships:
                membership_id = membership.get("membershipId")
                status = membership.get("membershipStatus")
                
                if status == "Active":
                    active_found = True
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=membership_id,
                        actual_value=f"Membership {membership_id} is Active",
                        remediation="No remediation needed"
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=membership_id,
                        actual_value=f"Membership {membership_id} status is {status}",
                        remediation="Activate the Security Incident Response membership through the AWS console"
                    ))
            
            if not active_found:
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="No active Security Incident Response memberships found",
                    remediation="Activate an existing membership or create a new active membership"
                ))

        return self.findings



"""
SRA-CONFIG-03: AWS Config Recording All Resource Types.
"""
from typing import List, Dict, Any
from sraverify.services.config.base import ConfigCheck
from sraverify.core.logging import logger


class SRA_CONFIG_03(ConfigCheck):
    """Check if AWS Config latest recording event is processed successfully."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CONFIG-03"
        self.check_name = "AWS Config latest recording event is processed successfully"
        self.account_type = "application"  # This check applies to all account types
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether the last delivery attempt to the delivery channel was successful "
            "to ensure you receive configuration change notifications. As AWS Config continually records "
            "the changes that occur to your AWS resources, it sends notifications and updated configuration "
            "states through the delivery channel."
        )
        self.check_logic = (
            "Checks if the lastStatus of the delivery channel is SUCCESS."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        if not self.regions:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id="config:global",
                    actual_value="No regions specified for check",
                    remediation="Specify at least one region when running the check"
                )
            )
            return findings
        
        # Check each region for delivery channel status
        for region in self.regions:
            # Get delivery channels for the region
            channels = self.get_delivery_channels(region)
            
            # Get delivery channel status for the region
            channel_statuses = self.get_delivery_channel_status(region)
            
            if not channels:
                # No delivery channel found in this region
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"arn:aws:config:{region}:{self.account_id}:deliveryChannel/default",
                        actual_value="No delivery channel found in this region",
                        remediation=(
                            f"Create a delivery channel in {region} using: aws configservice put-delivery-channel --delivery-channel name=default,s3BucketName=config-bucket-{self.account_id},snsTopicARN=arn:aws:sns:{region}:{self.account_id}:config-notifications --region {region}. "
                            f"Note: You must first create an S3 bucket and SNS topic with appropriate permissions for AWS Config."
                        )
                    )
                )
                continue
            
            if not channel_statuses:
                # No delivery channel status found in this region
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"arn:aws:config:{region}:{self.account_id}:deliveryChannel/default",
                        actual_value="No delivery channel status found in this region",
                        remediation=(
                            f"Check the delivery channel configuration in {region} and ensure it's properly configured. "
                            f"Verify S3 bucket permissions and SNS topic permissions are correctly set up for AWS Config."
                        )
                    )
                )
                continue
            
            # Check each delivery channel status
            for status in channel_statuses:
                channel_name = status.get('name', 'default')
                resource_id = f"config:deliveryChannel:{channel_name}"
                
                # Check configHistoryDeliveryInfo - this is the most important one
                history_info = status.get('configHistoryDeliveryInfo', {})
                history_status = history_info.get('lastStatus', 'UNKNOWN')
                history_attempt_time = history_info.get('lastAttemptTime', 'UNKNOWN')
                history_success_time = history_info.get('lastSuccessfulTime', 'UNKNOWN')
                
                # Check configSnapshotDeliveryInfo - may be UNKNOWN if no snapshot has been taken yet
                snapshot_info = status.get('configSnapshotDeliveryInfo', {})
                snapshot_status = snapshot_info.get('lastStatus', 'UNKNOWN')
                
                # Check configStreamDeliveryInfo - may be NOT_APPLICABLE if streaming is not configured
                stream_info = status.get('configStreamDeliveryInfo', {})
                stream_status = stream_info.get('lastStatus', 'UNKNOWN')
                
                # Determine overall status - focus on history delivery as the primary indicator
                # Stream may be NOT_APPLICABLE and snapshot may be UNKNOWN if not configured/used
                if history_status == "SUCCESS":
                    # History delivery is successful, which is the most important
                    findings.append(
                        self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=resource_id,
                            actual_value=(
                                f"Delivery channel '{channel_name}' is processing events successfully: "
                                f"Config History: {history_status} (Last success: {history_success_time}), "
                                f"Config Snapshot: {snapshot_status}, "
                                f"Config Stream: {stream_status}"
                            ),
                            remediation="No remediation needed"
                        )
                    )
                else:
                    # History delivery is not successful
                    findings.append(
                        self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=resource_id,
                            actual_value=(
                                f"Delivery channel '{channel_name}' has issues processing events: "
                                f"Config History: {history_status} (Last attempt: {history_attempt_time}), "
                                f"Config Snapshot: {snapshot_status}, "
                                f"Config Stream: {stream_status}"
                            ),
                            remediation=(
                                "Check the following: 1. S3 bucket permissions - ensure Config has write access. "
                                "2. SNS topic permissions - ensure Config can publish to the topic. "
                                "3. IAM role permissions - ensure Config service role has necessary permissions. "
                                "4. Check CloudWatch Logs for Config service errors."
                            )
                        )
                    )
        
        return findings



"""
SRA-SECURITYHUB-04: Security Hub check.
"""
from typing import List, Dict, Any
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_04(SecurityHubCheck):
    """Check if Security Hub central configuration is enabled."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-04"
        self.check_name = "Security Hub central configuration is enabled"
        self.account_type = "audit"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether Security Hub is configured for central configuration. "
            "Central configuration allows the delegated administrator to manage Security Hub, "
            "standards, and controls across all organization accounts from a single location."
        )
        self.check_logic = (
            "Check evaluates if Security Hub organization configuration has ConfigurationType "
            "set to CENTRAL and Status set to ENABLED in the delegated administrator account in all regions."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        for region in self.regions:
            org_config = self.get_organization_configuration(region)
            
            org_configuration = org_config.get('OrganizationConfiguration', {})
            config_type = org_configuration.get('ConfigurationType')
            status = org_configuration.get('Status')
            
            resource_id = f"securityhub:hub/{self.account_id}"
            
            if config_type != 'CENTRAL' or status != 'ENABLED':
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="OrganizationConfiguration Configuration Type is Central and Status Enabled",
                        actual_value=f"Security Hub delegated admin {self.account_id} is not setup properly to view findings for associated member accounts via ConfigurationType:{config_type} and Status:{status} in region {region}",
                        remediation=(
                            "Configure Security Hub with central configuration. In the Security Hub delegated admin account, "
                            "navigate to Settings > General > Configuration and select 'Centrally manage Security Hub across all accounts in your organization'. "
                            "Alternatively, use the AWS CLI command: "
                            f"aws securityhub update-organization-configuration --organization-configuration ConfigurationType=CENTRAL --region {region}"
                        )
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="OrganizationConfiguration Configuration Type is Central and Status Enabled",
                        actual_value=f"Security Hub delegated admin {self.account_id} is setup properly to view findings for associated member accounts via ConfigurationType:{config_type} and Status:{status} in region {region}",
                        remediation="No remediation needed"
                    )
                )
        
        return findings



from typing import Dict, List, Any
from sraverify.services.waf.base import WAFCheck

class SRA_WAF_03(WAFCheck):
    def __init__(self):
        super().__init__()
        self.resource_type = "AWS::ApiGateway::RestApi"
        self.check_id = "SRA-WAF-03"
        self.check_name = "API Gateway REST APIs should be associated with AWS WAF"
        self.description = "Ensures that all API Gateway REST APIs are protected by AWS WAF web ACLs to filter malicious traffic"
        self.severity = "HIGH"
        self.check_logic = "Lists all API Gateway REST APIs and verifies each has a WAF web ACL associated"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            rest_apis_response = self.get_rest_apis(region)

            if "Error" in rest_apis_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=rest_apis_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for API Gateway and WAF API access"
                ))
                continue

            rest_apis = rest_apis_response.get("items", [])

            if not rest_apis:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="No REST APIs",
                    actual_value="No API Gateway REST APIs found",
                    remediation="No action needed"
                ))
                continue

            for api in rest_apis:
                api_id = api.get("id")
                api_name = api.get("name")
                
                # Get all stages for this API
                stages_response = self.get_stages(region, api_id)
                
                if "Error" in stages_response:
                    self.findings.append(self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=api_name or api_id,
                        actual_value=stages_response["Error"].get("Message", "Unknown error"),
                        remediation="Check IAM permissions for API Gateway access"
                    ))
                    continue
                
                stages = stages_response.get("item", [])
                
                if not stages:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=api_name or api_id,
                        actual_value="No stages deployed",
                        remediation="Deploy the API to a stage and associate a WAF Web ACL"
                    ))
                    continue
                
                # Check each stage for WAF association
                for stage in stages:
                    stage_name = stage.get("stageName")
                    resource_id = f"{api_name or api_id}/{stage_name}"
                    
                    # Construct the API Gateway stage ARN for WAF association check
                    api_arn = f"arn:aws:apigateway:{region}::/restapis/{api_id}/stages/{stage_name}"
                    
                    client = self.get_client(region)
                    if not client:
                        continue

                    web_acl_response = client.get_web_acl_for_resource(api_arn)

                    if "Error" in web_acl_response:
                        error_code = web_acl_response["Error"].get("Code")
                        if error_code == "AccessDeniedException":
                            self.findings.append(self.create_finding(
                                status="ERROR",
                                region=region,
                                resource_id=resource_id,
                                actual_value=web_acl_response["Error"].get("Message", "Access denied"),
                                remediation="Check IAM permissions for wafv2:GetWebACLForResource"
                            ))
                        else:
                            self.findings.append(self.create_finding(
                                status="FAIL",
                                region=region,
                                resource_id=resource_id,
                                actual_value="No WAF Web ACL associated",
                                remediation="Associate a WAF Web ACL with this API Gateway stage using the AWS Console, CLI, or API"
                            ))
                        continue

                    web_acl = web_acl_response.get("WebACL")

                    if web_acl:
                        web_acl_name = web_acl.get("Name", "Unknown")
                        self.findings.append(self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=resource_id,
                            actual_value=f"WAF Web ACL associated: {web_acl_name}",
                            remediation="No action needed"
                        ))
                    else:
                        self.findings.append(self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=resource_id,
                            actual_value="No WAF Web ACL associated",
                            remediation="Associate a WAF Web ACL with this API Gateway stage using the AWS Console, CLI, or API"
                        ))

        return self.findings



"""
SRA-SECURITYHUB-10: Security Hub check.
"""
from typing import List, Dict, Any
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_10(SecurityHubCheck):
    """Check if Security Hub auto-enable is configured for new member accounts."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-10"
        self.check_name = "Security Hub auto-enable is configured"
        self.account_type = "audit"  # This check is for the audit account
        self.severity = "MEDIUM"
        self.description = (
            "This check verifies whether Security Hub is configured to be automatically enabled "
            "for new member accounts when they join the organization."
        )
        self.check_logic = (
            "Check evaluates Security Hub organization configuration. For central configuration, "
            "PASS if ConfigurationType is CENTRAL (uses configuration policies). For local configuration, "
            "PASS if AutoEnable is true."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check each region separately
        for region in self.regions:
            # Get organization configuration in this specific region
            org_config = self.get_organization_configuration(region)
            
            resource_id = f"securityhub:organization-configuration/{self.account_id}"
            
            # Check if using central configuration
            org_configuration = org_config.get('OrganizationConfiguration', {})
            config_type = org_configuration.get('ConfigurationType')
            
            if config_type == 'CENTRAL':
                # In central configuration, AutoEnable is always false and not relevant
                # Configuration policies handle new account enablement
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="Security Hub auto-enable configured for new accounts",
                        actual_value=f"Central configuration enabled [ConfigurationType: CENTRAL] in region {region} - new accounts managed via configuration policies",
                        remediation="No remediation needed - central configuration manages new account enablement through configuration policies"
                    )
                )
            else:
                # For local configuration, check AutoEnable
                auto_enable = org_config.get('AutoEnable', False)
                
                if not auto_enable:
                    findings.append(
                        self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=resource_id,
                            checked_value="Security Hub is set to auto-enable for new member accounts",
                            actual_value=f"AutoEnable is set to false in region {region}",
                            remediation=(
                                f"Configure Security Hub to automatically enable for new member accounts in region {region}. "
                                f"In the AWS Console, navigate to Security Hub in region {region}, go to Settings > Configuration, "
                                f"and enable 'Auto-enable Security Hub for new accounts'. Alternatively, use the AWS CLI command: "
                                f"aws securityhub update-organization-configuration --auto-enable --region {region}"
                            )
                        )
                    )
                else:
                    findings.append(
                        self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=resource_id,
                            checked_value="Security Hub is set to auto-enable for new member accounts",
                            actual_value=f"AutoEnable is set to true in region {region}",
                            remediation="No remediation needed"
                        )
                    )
        
        return findings



"""
Output handling for sraverify scan results.
"""
import csv
from typing import List, Dict, Any

# Required fields as per developer guide
REQUIRED_FIELDS = [
    'AccountId',
    'AccountName',
    'Region',
    'CheckId',
    'Status',
    'Severity',
    'Title',
    'Description',
    'ResourceId',
    'ResourceType',
    'CheckedValue',
    'ActualValue',
    'Remediation',
    'Service',
    'CheckLogic',
    'AccountType'  # Changed from CheckType to AccountType
]


def write_csv_output(findings: List[Dict[str, Any]], output_file: str):
    """
    Write scan findings to a CSV file ensuring all required fields are present.
    
    Args:
        findings: List of finding dictionaries
        output_file: Path to output CSV file
    """
    # Always create the CSV file, even if there are no findings
    if not findings:
        findings = []
    
    # Ensure each finding has all required fields
    for finding in findings:
        for field in REQUIRED_FIELDS:
            if field not in finding:
                finding[field] = ''  # Add empty string for missing fields
        
        # Handle the case where a finding might have CheckType but not AccountType
        if 'CheckType' in finding and 'AccountType' not in finding:
            finding['AccountType'] = finding['CheckType']
            del finding['CheckType']
    
    with open(output_file, 'w', newline='') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=REQUIRED_FIELDS)
        writer.writeheader()
        for finding in findings:
            # Only write the required fields, in the correct order
            row = {field: finding.get(field, '') for field in REQUIRED_FIELDS}
            writer.writerow(row)



"""
Check if organization has Resource Control Policies configured.
"""
from typing import Dict, List, Any
from sraverify.services.organizations.base import OrganizationsCheck


class SRA_ORGANIZATIONS_07(OrganizationsCheck):
    """Check if organization has Resource Control Policies configured."""

    def __init__(self):
        """Initialize RCP check."""
        super().__init__(resource_type="AWS::Organizations::Policy")
        self.check_id = "SRA-ORGANIZATIONS-07"
        self.check_name = "Organization has Resource Control Policies configured"
        self.description = (
            "This check verifies that the organization has at least one custom Resource Control Policy (RCP) "
            "configured beyond the default RCPFullAWSAccess policy. RCPs are a type of organization policy "
            "that help you centrally establish data perimeter controls across AWS resources in your organization. "
            "RCPs complement SCPs by controlling what resources can be accessed rather than what actions "
            "principals can perform."
        )
        self.severity = "MEDIUM"
        self.check_logic = (
            "Call ListPolicies API with filter for RESOURCE_CONTROL_POLICY type. "
            "Check passes if at least one custom RCP exists (AwsManaged=False), "
            "fails if only the default RCPFullAWSAccess policy exists or no RCPs are found."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Organizations is a global service, use "global" as region
        region = "global"

        # Get organization details for org_id
        org_response = self.get_organization()
        org_id = None
        if "Organization" in org_response:
            org_id = org_response["Organization"].get("Id", "Unknown")

        # Get RCPs
        response = self.list_policies("RESOURCE_CONTROL_POLICY")

        # Check for errors
        if "Error" in response:
            error_code = response["Error"].get("Code", "")
            error_message = response["Error"].get("Message", "Unknown error")

            # PolicyTypeNotEnabledException means RCPs are not enabled
            if error_code == "PolicyTypeNotEnabledException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=org_id,
                    actual_value="Resource Control Policies are not enabled",
                    remediation=(
                        "Enable Resource Control Policies in AWS Organizations. "
                        "Navigate to AWS Organizations > Policies > Resource control policies and enable RCPs. "
                        "Then create RCPs to implement data perimeter controls across your organization."
                    ),
                    checked_value="RCPs configured"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=org_id,
                    actual_value=f"Error: {error_message}",
                    remediation="Check IAM permissions for Organizations API access",
                    checked_value="RCPs configured"
                ))
            return self.findings

        policies = response.get("Policies", [])

        # Count custom RCPs (not AWS managed - exclude RCPFullAWSAccess)
        custom_rcps = [p for p in policies if not p.get("AwsManaged", False)]
        custom_rcp_count = len(custom_rcps)

        if not policies:
            # No RCPs at all - RCPs might not be enabled
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=org_id,
                actual_value="No Resource Control Policies found",
                remediation=(
                    "Enable Resource Control Policies in AWS Organizations and create RCPs. "
                    "Navigate to AWS Organizations > Policies > Resource control policies and enable RCPs. "
                    "Then create RCPs to implement data perimeter controls such as restricting access "
                    "to resources based on organization membership or network location."
                ),
                checked_value="Custom RCPs configured"
            ))
        elif custom_rcp_count == 0:
            # Only AWS managed policies (RCPFullAWSAccess)
            policy_names = [p.get("Name", "Unknown") for p in policies]
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=org_id,
                actual_value=f"Only default policies found: {', '.join(policy_names)}",
                remediation=(
                    "Create custom Resource Control Policies to implement data perimeter controls. "
                    "Navigate to AWS Organizations > Policies > Resource control policies and create new RCPs. "
                    "Consider implementing RCPs for: restricting access to resources based on organization "
                    "membership, enforcing HTTPS connections, and cross-service confused deputy protection."
                ),
                checked_value="Custom RCPs configured"
            ))
        else:
            # Custom RCPs exist
            custom_rcp_names = [p.get("Name", "Unknown") for p in custom_rcps]
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id=org_id,
                actual_value=f"{custom_rcp_count} custom RCP(s) configured: {', '.join(custom_rcp_names)}",
                remediation="No remediation needed",
                checked_value="Custom RCPs configured"
            ))

        return self.findings



from sraverify.services.securityincidentresponse.checks.sra_securityincidentresponse_01 import SRA_SECURITYINCIDENTRESPONSE_01
from sraverify.services.securityincidentresponse.checks.sra_securityincidentresponse_02 import SRA_SECURITYINCIDENTRESPONSE_02
from sraverify.services.securityincidentresponse.checks.sra_securityincidentresponse_03 import SRA_SECURITYINCIDENTRESPONSE_03
from sraverify.services.securityincidentresponse.checks.sra_securityincidentresponse_04 import SRA_SECURITYINCIDENTRESPONSE_04
from sraverify.services.securityincidentresponse.checks.sra_securityincidentresponse_05 import SRA_SECURITYINCIDENTRESPONSE_05

CHECKS = {
    "SRA-SECURITYINCIDENTRESPONSE-01": SRA_SECURITYINCIDENTRESPONSE_01,
    "SRA-SECURITYINCIDENTRESPONSE-02": SRA_SECURITYINCIDENTRESPONSE_02,
    "SRA-SECURITYINCIDENTRESPONSE-03": SRA_SECURITYINCIDENTRESPONSE_03,
    "SRA-SECURITYINCIDENTRESPONSE-04": SRA_SECURITYINCIDENTRESPONSE_04,
    "SRA-SECURITYINCIDENTRESPONSE-05": SRA_SECURITYINCIDENTRESPONSE_05,
}



"""
SRA-SECURITYHUB-11: Security Hub check.
"""
from typing import List, Dict, Any
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_11(SecurityHubCheck):
    """Check if Security Hub member account limit has not been reached."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-11"
        self.check_name = "Security Hub member account limit not reached"
        self.account_type = "audit"  # This check is for the audit account
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether the maximum number of allowed member accounts are already associated "
            "with the delegated administrator account for the AWS Organization."
        )
        self.check_logic = (
            "Check evaluates if Security Hub describe-organization-configuration returns \"MemberAccountLimitReached\": false. "
            "PASS if MemberAccountLimitReached is false."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check each region separately
        for region in self.regions:
            # Get organization configuration in this specific region
            org_config = self.get_organization_configuration(region)
            
            resource_id = f"securityhub:member-quota/{self.account_id}"
            
            # Check if MemberAccountLimitReached is false
            limit_reached = org_config.get('MemberAccountLimitReached', True)
            
            if limit_reached:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="Security Hub has not hit member account limit",
                        actual_value=f"Security Hub has hit member account limit in region {region}",
                        remediation=(
                            f"Contact AWS Support to request an increase in the Security Hub member account limit for region {region}. "
                            f"Alternatively, review your Security Hub member accounts and consider removing inactive or unnecessary accounts."
                        )
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="Security Hub has not hit member account limit",
                        actual_value=f"Security Hub has not hit member account limit in region {region}",
                        remediation="No remediation needed"
                    )
                )
        
        return findings



"""
Organizations client for interacting with AWS Organizations service.
"""
from typing import Dict, List, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class OrganizationsClient:
    """Client for interacting with AWS Organizations service."""
    
    def __init__(self, session: Optional[boto3.Session] = None):
        """
        Initialize Organizations client.
        
        Args:
            session: AWS session to use (if None, a new session will be created)
        """
        self.session = session or boto3.Session()
        # Organizations is a global service, always use us-east-1
        self.client = self.session.client('organizations', region_name='us-east-1')
    
    def describe_organization(self) -> Dict[str, Any]:
        """
        Get organization details.
        
        Returns:
            Dictionary with Organization key containing organization details,
            or Error key if an error occurred.
        """
        try:
            response = self.client.describe_organization()
            return response
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = e.response.get('Error', {}).get('Message', str(e))
            logger.error(f"Error describing organization: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }

    def list_roots(self) -> Dict[str, Any]:
        """
        List organization roots with pagination support.
        
        Returns:
            Dictionary with Roots key containing list of roots,
            or Error key if an error occurred.
        """
        try:
            roots = []
            paginator = self.client.get_paginator('list_roots')
            for page in paginator.paginate():
                roots.extend(page.get('Roots', []))
            return {"Roots": roots}
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = e.response.get('Error', {}).get('Message', str(e))
            logger.error(f"Error listing roots: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }
    
    def list_organizational_units_for_parent(self, parent_id: str) -> Dict[str, Any]:
        """
        List organizational units under a parent with pagination support.
        
        Args:
            parent_id: The ID of the parent root or OU
            
        Returns:
            Dictionary with OrganizationalUnits key containing list of OUs,
            or Error key if an error occurred.
        """
        try:
            ous = []
            paginator = self.client.get_paginator('list_organizational_units_for_parent')
            for page in paginator.paginate(ParentId=parent_id):
                ous.extend(page.get('OrganizationalUnits', []))
            return {"OrganizationalUnits": ous}
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = e.response.get('Error', {}).get('Message', str(e))
            logger.error(f"Error listing OUs for parent {parent_id}: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }
    
    def list_policies(self, policy_type: str = "SERVICE_CONTROL_POLICY") -> Dict[str, Any]:
        """
        List policies by type with pagination support.
        
        Args:
            policy_type: Type of policy to list (default: SERVICE_CONTROL_POLICY)
            
        Returns:
            Dictionary with Policies key containing list of policies,
            or Error key if an error occurred.
        """
        try:
            policies = []
            paginator = self.client.get_paginator('list_policies')
            for page in paginator.paginate(Filter=policy_type):
                policies.extend(page.get('Policies', []))
            return {"Policies": policies}
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = e.response.get('Error', {}).get('Message', str(e))
            logger.error(f"Error listing policies of type {policy_type}: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }

    def list_accounts_for_parent(self, parent_id: str) -> Dict[str, Any]:
        """
        List accounts under a parent (root or OU) with pagination support.
        
        Args:
            parent_id: The ID of the parent root or OU
            
        Returns:
            Dictionary with Accounts key containing list of accounts,
            or Error key if an error occurred.
        """
        try:
            accounts = []
            paginator = self.client.get_paginator('list_accounts_for_parent')
            for page in paginator.paginate(ParentId=parent_id):
                accounts.extend(page.get('Accounts', []))
            return {"Accounts": accounts}
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            error_message = e.response.get('Error', {}).get('Message', str(e))
            logger.error(f"Error listing accounts for parent {parent_id}: {error_message}")
            return {
                "Error": {
                    "Code": error_code,
                    "Message": error_message
                }
            }



"""
SRA-CONFIG-05: AWS Config Recorder Status.
"""
from typing import List, Dict, Any
from sraverify.services.config.base import ConfigCheck
from sraverify.core.logging import logger


class SRA_CONFIG_05(ConfigCheck):
    """Check if AWS Config organization aggregator includes all regions."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CONFIG-05"
        self.check_name = "AWS Config organization aggregator includes all regions"
        self.account_type = "audit"  # This check applies to audit account
        self.severity = "MEDIUM"
        self.description = (
            "This check verifies that the AWS Config organization aggregator is configured to aggregate "
            "config data from all existing and future AWS Regions. This provides you visibility into "
            "activities across all regions even if your business does not operate in the region."
        )
        self.check_logic = (
            "Checks if AWS Config organization aggregator has AllAwsRegions set to true."
        )
        self.resource_type = "AWS::Config::ConfigurationAggregator"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        if not self.regions:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id="config:global",
                    checked_value="AllAwsRegions: true",
                    actual_value="No regions specified for check",
                    remediation="Specify at least one region when running the check"
                )
            )
            return findings
        
        # Check if an organization aggregator with AllAwsRegions=true exists in any region
        found_all_regions_aggregator = False
        all_regions_aggregator_region = None
        all_regions_aggregator_name = None
        all_regions_aggregator_arn = None
        
        for region in self.regions:
            # Get configuration aggregators for the region from cache
            aggregators = self.get_configuration_aggregators(region)
            
            # Check if any of the aggregators is an organization aggregator with all regions enabled
            for aggregator in aggregators:
                if ('OrganizationAggregationSource' in aggregator and 
                    aggregator.get('OrganizationAggregationSource', {}).get('AllAwsRegions', False)):
                    found_all_regions_aggregator = True
                    all_regions_aggregator_region = region
                    all_regions_aggregator_name = aggregator.get('ConfigurationAggregatorName', 'Unknown')
                    all_regions_aggregator_arn = aggregator.get('ConfigurationAggregatorArn', 
                                                              f"arn:aws:config:{region}:{self.account_id}:config-aggregator/{all_regions_aggregator_name}")
                    break
            
            if found_all_regions_aggregator:
                break
        
        # Return a single finding based on whether an organization aggregator with AllAwsRegions=true was found
        if found_all_regions_aggregator:
            findings.append(
                self.create_finding(
                    status="PASS",
                    region="global",
                    resource_id=all_regions_aggregator_arn,
                    checked_value="AllAwsRegions: true",
                    actual_value=f"Configuration aggregator '{all_regions_aggregator_name}' is configured to aggregate all regions, located in region {all_regions_aggregator_region} with Region selection \"All current and future AWS regions\"",
                    remediation="No remediation needed"
                )
            )
        else:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=f"arn:aws:config:global:{self.account_id}:config-aggregator/none",
                    checked_value="AllAwsRegions: true",
                    actual_value="No organization aggregator with AllAwsRegions=true found in any region",
                    remediation=(
                        "Create an organization configuration aggregator with AllAwsRegions=true in at least one region using: aws configservice put-configuration-aggregator "
                        "--configuration-aggregator-name organization-aggregator --organization-aggregation-source "
                        f"\"EnableAllRegions=true,RoleArn=arn:aws:iam::{self.account_id}:role/aws-service-role/config.amazonaws.com/AWSServiceRoleForConfigServiceRole\" "
                        "--region <region>"
                    )
                )
            )
        
        return findings



"""
Check if Shield Response Team (SRT) access is configured.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_08(ShieldCheck):
    """Check if Shield Response Team (SRT) access is configured."""

    def __init__(self):
        """Initialize Shield Response Team access check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-08"
        self.check_name = "Shield Response Team (SRT) access is configured"
        self.description = ("This check verifies that AWS Shield Response Team (SRT) "
                            "access is configured with an appropriate IAM role.")
        self.severity = "MEDIUM"
        self.check_logic = "Describe DRT access configuration. Check fails if no role ARN is configured."

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        drt_access = self.describe_drt_access(region)

        if "Error" in drt_access:
            error_code = drt_access["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="SRT access not configured",
                    remediation="Configure Shield Response Team access by associating an IAM role using AssociateDRTRole API"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=drt_access["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif drt_access.get("RoleArn"):
            role_arn = drt_access["RoleArn"]
            bucket_count = len(drt_access.get("LogBucketList", []))
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id="shield:srt-access",
                actual_value=f"SRT access configured with role: {role_arn}, {bucket_count} log bucket(s)",
                remediation=""
            ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="SRT access not configured",
                remediation="Configure Shield Response Team access by associating an IAM role using AssociateDRTRole API"
            ))

        return self.findings



"""
Check if GuardDuty detector exists.
"""
from typing import Dict, List, Any
from sraverify.services.guardduty.base import GuardDutyCheck


class SRA_GUARDDUTY_01(GuardDutyCheck):
    """Check if GuardDuty detector exists."""

    def __init__(self):
        """Initialize GuardDuty enabled check."""
        super().__init__()
        self.check_id = "SRA-GUARDDUTY-01"
        self.check_name = "GuardDuty detector exists"
        self.description = "This check verifies that an GuardDuty detector exists in the AWS Region.\
              A detector is a resource that represents the GuardDuty service and should be present \
                in all AWS member account and AWS Region so that GuardDuty can generate findings \
                    about unauthorized or unusual activity even in those Regions that you may not \
                        be using actively."
        self.severity = "HIGH"
        self.check_logic = "Get detector_id in each Region. Check fails if there is no detector_id"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        for region in self.regions:
            detector_id = self.get_detector_id(region)
            
            if not detector_id:
                self.findings.append(self.create_finding(
                    status="FAIL", 
                    region=region, 
                    resource_id=None, 
                    actual_value=None, 
                    remediation=f"Enable GuardDuty in {region}"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="PASS", 
                    region=region, 
                    resource_id=f"guardduty:{region}:{detector_id}", 
                    actual_value=None, 
                    remediation=""
                ))
        
        return self.findings



"""
Check if AWS Organizations is enabled.
"""
from typing import Dict, List, Any
from sraverify.services.organizations.base import OrganizationsCheck


class SRA_ORGANIZATIONS_01(OrganizationsCheck):
    """Check if AWS Organizations is enabled."""

    def __init__(self):
        """Initialize Organizations enabled check."""
        super().__init__(resource_type="AWS::Organizations::Organization")
        self.check_id = "SRA-ORGANIZATIONS-01"
        self.check_name = "AWS Organizations is enabled"
        self.description = (
            "This check verifies that AWS Organizations is enabled for the account. "
            "AWS Organizations enables central management and governance of multiple AWS accounts, "
            "providing consolidated billing, account management, and policy-based controls."
        )
        self.severity = "HIGH"
        self.check_logic = (
            "Call DescribeOrganization API to confirm an organization exists. "
            "Check passes if an organization is found, fails if no organization exists."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        # Organizations is a global service, use "global" as region
        region = "global"
        
        # Get organization details
        response = self.get_organization()
        
        # Check for errors
        if "Error" in response:
            error_code = response["Error"].get("Code", "")
            error_message = response["Error"].get("Message", "Unknown error")
            
            # AWSOrganizationsNotInUseException means no organization exists
            if error_code == "AWSOrganizationsNotInUseException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="No organization exists",
                    remediation=(
                        "Create an AWS Organization by navigating to AWS Organizations in the console "
                        "and clicking 'Create organization', or use the AWS CLI command: "
                        "aws organizations create-organization"
                    ),
                    checked_value="AWS Organizations enabled"
                ))
            else:
                # Other errors (permissions, service errors)
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=f"Error: {error_message}",
                    remediation="Check IAM permissions for Organizations API access",
                    checked_value="AWS Organizations enabled"
                ))
            return self.findings
        
        # Organization exists - extract details
        organization = response.get("Organization", {})
        org_id = organization.get("Id", "Unknown")
        
        self.findings.append(self.create_finding(
            status="PASS",
            region=region,
            resource_id=org_id,
            actual_value=f"Organization exists: {org_id}",
            remediation="No remediation needed",
            checked_value="AWS Organizations enabled"
        ))
        
        return self.findings



"""
Check if IAM Access Analyzer delegated admin is the Audit account.
"""
from typing import Dict, List, Any
from sraverify.services.accessanalyzer.base import AccessAnalyzerCheck
from sraverify.core.logging import logger


class SRA_ACCESSANALYZER_03(AccessAnalyzerCheck):
    """Check if IAM Access Analyzer delegated admin is the Audit account."""
    
    def __init__(self):
        """Initialize IAM Access Analyzer check."""
        super().__init__()
        self.check_id = "SRA-ACCESSANALYZER-03"
        self.check_name = "IAM Access Analyzer Delegated Admin is the Audit Account"
        self.description = ("This check verifies whether IAA delegated admin account is the "
                          "audit account of your AWS organization. Audit account is "
                          "dedicated to operating security services, monitoring AWS accounts, and "
                          "automating security alerting and response. IAA helps monitor resources "
                          "shared outside zone of trust.")
        self.severity = "HIGH"
        self.account_type = "management"
        self.check_logic = ("Check if the delegated administrator account matches any of the specified Audit account IDs")

    def execute(self) -> List[Dict[str, Any]]:
        """Execute the check."""
        findings = []        
        
        delegated_admin = self.get_delegated_admin()
        
        if not delegated_admin:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",                    
                    resource_id=f"organization/{self.account_id}",
                    actual_value="No delegated administrator configured for IAM Access Analyzer",
                    remediation="Configure a delegated administrator for IAM Access Analyzer first"
                )
            )
            return findings

        try:
            # Get the delegated admin account ID
            delegated_admin_id = delegated_admin.get('Id')
            
            # Check if audit_accounts is provided via _audit_accounts (new attribute name)
            audit_accounts = []
            if hasattr(self, '_audit_accounts') and self._audit_accounts:
                audit_accounts = self._audit_accounts
            # For backward compatibility, also check the old attribute name
            elif hasattr(self, 'audit_accounts') and self.audit_accounts:
                audit_accounts = self.audit_accounts
                
            if not audit_accounts:
                findings.append(
                    self.create_finding(
                        status="ERROR",
                        region="global",                        
                        resource_id=delegated_admin_id,
                        actual_value="Audit Account ID not provided",
                        remediation="Provide the Audit account IDs using --audit-account flag"
                    )
                )
                return findings
            
            # Check if delegated admin matches any of the specified Audit accounts
            if delegated_admin_id in audit_accounts:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region="global",                        
                        resource_id=delegated_admin_id,
                        actual_value=f"IAM Access Analyzer delegated administrator (Account: {delegated_admin_id}) "
                                   f"matches one of the specified Audit accounts {', '.join(audit_accounts)}",
                        remediation="No remediation needed"
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region="global",                        
                        resource_id=delegated_admin_id,
                        actual_value=f"IAM Access Analyzer delegated administrator (Account: {delegated_admin_id}) "
                                   f"does not match any of the specified Audit accounts ({', '.join(audit_accounts)})",
                        remediation=f"Update the delegated administrator to be one of the Audit accounts ({', '.join(audit_accounts)})"
                    )
                )
                
        except Exception as e:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",                    
                    resource_id=delegated_admin_id if 'delegated_admin_id' in locals() else f"organization/{self.account_id}",
                    actual_value=f"Error checking delegated administrator: {str(e)}",
                    remediation="Ensure proper permissions to check Organizations structure"
                )
            )

        return findings



from typing import Dict, List, Any
from sraverify.services.waf.base import WAFCheck

class SRA_WAF_04(WAFCheck):
    def __init__(self):
        super().__init__()
        self.resource_type = "AWS::AppSync::GraphQLApi"
        self.check_id = "SRA-WAF-04"
        self.check_name = "AppSync GraphQL APIs should be associated with AWS WAF"
        self.description = "Ensures that all AppSync GraphQL APIs are protected by AWS WAF web ACLs to filter malicious traffic"
        self.severity = "HIGH"
        self.check_logic = "Lists all AppSync GraphQL APIs and verifies each has a WAF web ACL associated"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            graphql_apis_response = self.get_graphql_apis(region)

            if "Error" in graphql_apis_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=graphql_apis_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for AppSync and WAF API access"
                ))
                continue

            graphql_apis = graphql_apis_response.get("graphqlApis", [])

            if not graphql_apis:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="No GraphQL APIs",
                    actual_value="No AppSync GraphQL APIs found",
                    remediation="No action needed"
                ))
                continue

            for api in graphql_apis:
                api_arn = api.get("arn")
                api_name = api.get("name")
                api_id = api.get("apiId")
                
                # Check if WAF is already associated (wafWebAclArn field)
                waf_web_acl_arn = api.get("wafWebAclArn")
                
                if waf_web_acl_arn:
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=api_name or api_id,
                        actual_value=f"WAF Web ACL associated: {waf_web_acl_arn}",
                        remediation="No action needed"
                    ))
                else:
                    # Double-check using WAF API
                    client = self.get_client(region)
                    if not client:
                        continue

                    web_acl_response = client.get_web_acl_for_resource(api_arn)

                    if "Error" in web_acl_response:
                        self.findings.append(self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=api_name or api_id,
                            actual_value="No WAF Web ACL associated",
                            remediation="Associate a WAF Web ACL with this AppSync GraphQL API using the AWS Console, CLI, or API"
                        ))
                        continue

                    web_acl = web_acl_response.get("WebACL")

                    if web_acl:
                        web_acl_name = web_acl.get("Name", "Unknown")
                        self.findings.append(self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=api_name or api_id,
                            actual_value=f"WAF Web ACL associated: {web_acl_name}",
                            remediation="No action needed"
                        ))
                    else:
                        self.findings.append(self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=api_name or api_id,
                            actual_value="No WAF Web ACL associated",
                            remediation="Associate a WAF Web ACL with this AppSync GraphQL API using the AWS Console, CLI, or API"
                        ))

        return self.findings



from typing import Dict, List, Any
from sraverify.services.securityincidentresponse.base import SecurityIncidentResponseCheck

class SRA_SECURITYINCIDENTRESPONSE_01(SecurityIncidentResponseCheck):
    def __init__(self):
        super().__init__()
        self.check_id = "SRA-SECURITYINCIDENTRESPONSE-01"
        self.check_name = "Security Incident Response delegated admin is audit account"
        self.description = "Verifies that the Security Incident Response delegated administrator is configured and is the audit account"
        self.severity = "HIGH"
        self.check_logic = "Lists delegated administrators for security-ir.amazonaws.com and verifies the audit account is designated"

    def execute(self) -> List[Dict[str, Any]]:
        region = self.regions[0] if self.regions else "us-east-1"
        
        # Check if audit accounts are provided
        audit_accounts = getattr(self, '_audit_accounts', None)
        if not audit_accounts:
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value="No audit accounts specified",
                remediation="Run with --audit-account parameter to specify audit account IDs"
            ))
            return self.findings
        
        response = self.get_delegated_administrators()
        
        if "Error" in response:
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value=response["Error"].get("Message", "Unknown error"),
                remediation="Check IAM permissions for Organizations API access"
            ))
            return self.findings

        delegated_admins = response.get("DelegatedAdministrators", [])
        
        if not delegated_admins:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No delegated administrator configured for Security Incident Response",
                remediation="Configure a delegated administrator for Security Incident Response using: aws organizations register-delegated-administrator --account-id <audit-account-id> --service-principal security-ir.amazonaws.com"
            ))
        else:
            # Check if any of the delegated admins is the audit account
            audit_admin_found = False
            
            for admin in delegated_admins:
                admin_id = admin.get("Id")
                if admin_id in audit_accounts:
                    audit_admin_found = True
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=admin_id,
                        actual_value=f"Audit account {admin_id} is configured as delegated administrator",
                        remediation="No remediation needed"
                    ))
                    break
            
            if not audit_admin_found:
                admin_ids = [admin.get("Id") for admin in delegated_admins]
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value=f"Delegated administrators found: {admin_ids}, but none are audit accounts: {audit_accounts}",
                    remediation="Register the audit account as delegated administrator: aws organizations register-delegated-administrator --account-id <audit-account-id> --service-principal security-ir.amazonaws.com"
                ))

        return self.findings



"""
SRA-EC2-01: AWS account level EBS encryption by default is enabled.
"""
from typing import List, Dict, Any
from sraverify.services.ec2.base import EC2Check
from sraverify.core.logging import logger


class SRA_EC2_01(EC2Check):
    """Check if AWS account level EBS encryption by default is enabled."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-EC2-01"
        self.check_name = "AWS account level EBS encryption by default is enabled"
        self.description = (
            "This check verifies that the AWS account level configuration to encrypt EBS volumes by default "
            "is enabled in the AWS Region. This enforces, at AWS account level, the encryption of the new EBS "
            "volumes and snapshot copies that you create. You can use AWS managed keys or a customer managed KMS key."
        )
        self.severity = "HIGH"
        self.account_type = "application"
        self.check_logic = "Check Pass if 'EbsEncryptionByDefault' = true."
        self.resource_type = "AWS::EC2::Volume"
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        for region in self.regions:
            # Get EBS encryption by default status using the base class method with caching
            encryption_status = self.get_ebs_encryption_by_default(region)
            
            # Check if the API call was successful
            if not encryption_status:
                findings.append(
                    self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=f"account/{self.account_id}/region/{region}",
                        checked_value="EbsEncryptionByDefault: true",
                        actual_value="Failed to retrieve EBS encryption by default status",
                        remediation="Ensure you have the necessary permissions to call the EC2 GetEbsEncryptionByDefault API"
                    )
                )
                continue
            
            # Check if EBS encryption by default is enabled
            is_encryption_enabled = encryption_status.get('EbsEncryptionByDefault', False)
            
            if is_encryption_enabled:
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"account/{self.account_id}/region/{region}",
                        checked_value="EbsEncryptionByDefault: true",
                        actual_value=f"EBS encryption by default is enabled in region {region}",
                        remediation="No remediation needed"
                    )
                )
            else:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"account/{self.account_id}/region/{region}",
                        checked_value="EbsEncryptionByDefault: true",
                        actual_value=f"EBS encryption by default is not enabled in region {region}",
                        remediation=(
                            f"Enable EBS encryption by default in region {region} using the AWS CLI command: "
                            f"aws ec2 enable-ebs-encryption-by-default --region {region}"
                        )
                    )
                )
        
        return findings
